from ...interface import IPacker


class SaveOHLCListParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [
            self._obj.MarketName,
            self._obj.ExchangeID,
            self._obj.InstrumentID,
            self._obj.Range,
            self._obj.TradingDay,
            self._obj.PreSettlementPrice,
            list(self._obj.ActionDayList),
            list(self._obj.ActionTimespanList),
            list(self._obj.TradingTimeList),
            list(self._obj.StartTimeList),
            list(self._obj.EndTimeList),
            list(self._obj.TotalTurnoverList),
            list(self._obj.OpenInterestList),
            list(self._obj.OpenPriceList),
            list(self._obj.OpenBidPriceList),
            list(self._obj.OpenAskPriceList),
            list(self._obj.OpenBidVolumeList),
            list(self._obj.OpenAskVolumeList),
            list(self._obj.HighPriceList),
            list(self._obj.HighBidPriceList),
            list(self._obj.HighAskPriceList),
            list(self._obj.HighBidVolumeList),
            list(self._obj.HighAskVolumeList),
            list(self._obj.LowerPriceList),
            list(self._obj.LowerBidPriceList),
            list(self._obj.LowerAskPriceList),
            list(self._obj.LowerBidVolumeList),
            list(self._obj.LowerAskVolumeList),
            list(self._obj.ClosePriceList),
            list(self._obj.CloseBidPriceList),
            list(self._obj.CloseAskPriceList),
            list(self._obj.CloseBidVolumeList),
            list(self._obj.CloseAskVolumeList),
        ]

    def tuple_to_obj(self, t):
        if len(t) >= 33:
            self._obj.MarketName = t[0]
            self._obj.ExchangeID = t[1]
            self._obj.InstrumentID = t[2]
            self._obj.Range = t[3]
            self._obj.TradingDay = t[4]
            self._obj.PreSettlementPrice = t[5]
            self._obj.ActionDayList = t[6]
            self._obj.ActionTimespanList = t[7]
            self._obj.TradingTimeList = t[8]
            self._obj.StartTimeList = t[9]
            self._obj.EndTimeList = t[10]
            self._obj.TotalTurnoverList = t[11]
            self._obj.OpenInterestList = t[12]
            self._obj.OpenPriceList = t[13]
            self._obj.OpenBidPriceList = t[14]
            self._obj.OpenAskPriceList = t[15]
            self._obj.OpenBidVolumeList = t[16]
            self._obj.OpenAskVolumeList = t[17]
            self._obj.HighPriceList = t[18]
            self._obj.HighBidPriceList = t[19]
            self._obj.HighAskPriceList = t[20]
            self._obj.HighBidVolumeList = t[21]
            self._obj.HighAskVolumeList = t[22]
            self._obj.LowerPriceList = t[23]
            self._obj.LowerBidPriceList = t[24]
            self._obj.LowerAskPriceList = t[25]
            self._obj.LowerBidVolumeList = t[26]
            self._obj.LowerAskVolumeList = t[27]
            self._obj.ClosePriceList = t[28]
            self._obj.CloseBidPriceList = t[29]
            self._obj.CloseAskPriceList = t[30]
            self._obj.CloseBidVolumeList = t[31]
            self._obj.CloseAskVolumeList = t[32]
            return True
        return False



from typing import Dict
from ...interface import IData
from ...packer.chart.marker_graph_param_data_packer import MarkerGraphParamDataPacker


class MarkerGraphParamData(IData):
    def __init__(self, id: str = '', name: str = '', plot_index: int = 0, value_axis_id: int = -1, text: str = '',
                 text_color: str = "white", text_v_align: int = 1, text_h_align: int = 1,
                 orientation: int = 1, line_direc: int = 1, line_width: int = 1, line_style: int = 1,
                 line_color: str = "white", key: float = 0.0, value: float = 0.0, mill_ts: int = -1,
                 join_value_axis=True, show_legend: bool = False, legend_format: str = '',
                 legend_color: str = '', user_data: Dict[str, str] = {}, visible: bool = True):
        """标记配置参数

        Args:
            id (str, optional): 图编码. Defaults to ''.
            name (str, optional): 图名称. Defaults to ''.
            plot_index (int, optional): 所属图块. Defaults to 0.
            value_axis_id (int, optional): 所属值轴. Defaults to -1.
            text (str, optional): 显示文字. Defaults to ''.
            text_color (str, optional): 文字颜色. Defaults to "white".
            text_v_align (int, optional): 文字垂直对齐方式 1.顶部 2.中间 3.底部 Defaults to 1.
            text_h_align (int, optional): 文字水平对齐方式 1.左侧 2.中间 3.右侧 Defaults to 1.
            orientation (int, optional): 文字方向 1.水平 2垂直 Defaults to 1.
            line_direc (int, optional): 线方向 1.水平 2.垂直. Defaults to 1.
            line_width (int, optional): 线宽度. Defaults to 1.
            line_style (int, optional): 线样式.1.实线 2.线状虚线 3.点状虚线 4.点线间隔线 5.点点线间隔线 Defaults to 1.
            line_color (str, optional): 线颜色. Defaults to "white".
            key (float, optional): 数值格式的x轴值. Defaults to 0.0.
            value (float, optional): 数值格式的y轴值. Defaults to 0.0.
            mill_ts (int, optional): 时间戳格式的x轴值. Defaults to -1.
            join_value_axis (bool, optional): 是否加入到y轴可视范围的计算中. Defaults to True.
            show_legend (bool, optional): 是否显示标签. Defaults to True.
            legend_format (str, optional): 标签格式. Defaults to ''.
            legend_color (str, optional): 标签颜色,默认与图颜色相同. Defaults to ''.
            user_data (Dict[str, str], optional): 用户自定义数据. Defaults to {}.
            visible (bool, optional): 是否显示 Defaults to True.
        """
        super().__init__(MarkerGraphParamDataPacker(self))
        self._ID: str = id
        self._Name: str = name
        self._PlotIndex: int = plot_index
        self._ValueAxisID: int = value_axis_id
        self._Text: str = text
        self._TextColor: str = text_color
        self._TextVAlign: int = text_v_align
        self._TextHAlign: int = text_h_align
        self._Orientation: int = orientation
        self._LineDirec: int = line_direc
        self._LineWidth: int = line_width
        self._LineStyle: int = line_style
        self._LineColor: str = line_color
        self._Key: float = key
        self._Value: float = value
        self._MillTimeSpan: int = mill_ts
        self._JoinValueAxis: bool = join_value_axis
        self._ShowLegend: bool = show_legend
        self._LegendFormat: str = legend_format
        self._LegendColor: str = legend_color
        self._UserData: Dict[str, str] = user_data
        self._Visible: bool = visible

    @property
    def ID(self):
        return self._ID

    @ID.setter
    def ID(self, value: str):
        self._ID = value

    @property
    def Name(self):
        return self._Name

    @Name.setter
    def Name(self, value: str):
        self._Name = value

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value: int):
        self._PlotIndex = value

    @property
    def ValueAxisID(self):
        return self._ValueAxisID

    @ValueAxisID.setter
    def ValueAxisID(self, value: int):
        self._ValueAxisID = value

    @property
    def Text(self):
        return self._Text

    @Text.setter
    def Text(self, value: str):
        self._Text = value

    @property
    def TextColor(self):
        return self._TextColor

    @TextColor.setter
    def TextColor(self, value: str):
        self._TextColor = value

    @property
    def TextVAlign(self):
        return self._TextVAlign

    @TextVAlign.setter
    def TextVAlign(self, value: int):
        self._TextVAlign = value

    @property
    def TextHAlign(self):
        return self._TextHAlign

    @TextHAlign.setter
    def TextHAlign(self, value: int):
        self._TextHAlign = value

    @property
    def Orientation(self):
        return self._Orientation

    @Orientation.setter
    def Orientation(self, value: int):
        self._Orientation = value

    @property
    def LineDirec(self):
        return self._LineDirec

    @LineDirec.setter
    def LineDirec(self, value: int):
        self._LineDirec = value

    @property
    def LineWidth(self):
        return self._LineWidth

    @LineWidth.setter
    def LineWidth(self, value: int):
        self._LineWidth = value

    @property
    def LineStyle(self):
        return self._LineStyle

    @LineStyle.setter
    def LineStyle(self, value: int):
        self._LineStyle = value

    @property
    def LineColor(self):
        return self._LineColor

    @LineColor.setter
    def LineColor(self, value: str):
        self._LineColor = value

    @property
    def Key(self):
        return self._Key

    @Key.setter
    def Key(self, value: float):
        self._Key = value

    @property
    def Value(self):
        return self._Value

    @Value.setter
    def Value(self, value: float):
        self._Value = value

    @property
    def MillTimeSpan(self):
        return self._MillTimeSpan

    @MillTimeSpan.setter
    def MillTimeSpan(self, value: int):
        self._MillTimeSpan = value

    @property
    def JoinValueAxis(self):
        return self._JoinValueAxis

    @JoinValueAxis.setter
    def JoinValueAxis(self, value: bool):
        self._JoinValueAxis = value

    @property
    def ShowLegend(self):
        return self._ShowLegend

    @ShowLegend.setter
    def ShowLegend(self, value):
        self._ShowLegend = (value)

    @property
    def LegendFormat(self):
        return self._LegendFormat

    @LegendFormat.setter
    def LegendFormat(self, value):
        self._LegendFormat = str(value)

    @property
    def LegendColor(self):
        return self._LegendColor

    @LegendColor.setter
    def LegendColor(self, value):
        self._LegendColor = str(value)

    @property
    def UserData(self):
        return self._UserData

    @UserData.setter
    def UserData(self, value: Dict[str, str]):
        self._UserData = value

    @property
    def Visible(self):
        return self._Visible

    @JoinValueAxis.setter
    def Visible(self, value: bool):
        self._Visible = value



from typing import List
from ...interface import IData
from ...packer.trade.read_history_order_param_data_packer import ReadHistoryOrderParamDataPacker
from .order_data import OrderData


class ReadHistoryOrderParamData(IData):
    def __init__(self, start_date: str = '', end_date: str = '', data_list: List[OrderData] = []):
        super().__init__(ReadHistoryOrderParamDataPacker(self))
        self._StartDate: str = start_date
        self._EndDate: str = end_date
        self._DataList: List[OrderData] = data_list

    @property
    def StartDate(self):
        return self._StartDate

    @StartDate.setter
    def StartDate(self, value: str):
        self._StartDate = value

    @property
    def EndDate(self):
        return self._EndDate

    @EndDate.setter
    def EndDate(self, value: str):
        self._EndDate = value

    @property
    def DataList(self):
        return self._DataList

    @DataList.setter
    def DataList(self, value: List[OrderData]):
        self._DataList = value



from ...interface import IData
from ...packer.chart.graph_value_data_packer import GraphValueDataPacker


class GraphValueData(IData):
    def __init__(self, graph_id: str = '', key: float = 0.0, mill_ts: int = -1, value: float = 0.0):
        """图数据

        Args:
            graph_id (str, optional): 图唯一编码. Defaults to ''.
            key (float, optional): 数值格式的X值. Defaults to 0.0.
            mill_ts (int, optional): 时间戳格式的X值. Defaults to -1.
            value (float, optional): Y值. Defaults to 0.0.
        """
        super().__init__(GraphValueDataPacker(self))
        self._GraphID = graph_id
        self._Key = key
        self._MillTimeSpan = mill_ts
        self._Value = value

    @property
    def GraphID(self):
        return self._GraphID

    @GraphID.setter
    def GraphID(self, value):
        self._GraphID = str(value)

    @property
    def Key(self):
        return self._Key

    @Key.setter
    def Key(self, value):
        self._Key = float(value)

    @property
    def MillTimeSpan(self):
        return self._MillTimeSpan

    @MillTimeSpan.setter
    def MillTimeSpan(self, value):
        self._MillTimeSpan = int(value)

    @property
    def Value(self):
        return self._Value

    @Value.setter
    def Value(self, value):
        self._Value = float(value)



from typing import Dict, List
import threading
from ..data.message_data import MessageData



class ReqRspDict():
    __Lock = threading.Lock()

    def __init__(self):
        self.__req_rsp_dict: Dict[str, ReqRsp] = {}

    def remove(self, key: str):
        self.__Lock.acquire()
        del self.__req_rsp_dict[key]
        self.__Lock.release()

    def new_reqrsp(self, key: str, req_msg: MessageData):
        self.__Lock.acquire()
        req_rsp = ReqRsp(req_msg)
        self.__req_rsp_dict[key] = req_rsp
        self.__Lock.release()

        return req_rsp

    def get_reqrsp(self, key: str):
        self.__Lock.acquire()
        reqrsp = self.__req_rsp_dict.get(key)
        self.__Lock.release()
        return reqrsp


class ReqRsp():
    __Event = threading.Event()

    def __init__(self, req_msg: MessageData):
        self.__req_message: MessageData = req_msg
        self.__rsp_last_msg: MessageData = None
        self.__rsp_msg_list: List[MessageData] = []

    def append_rsp(self, rsp_msg: MessageData):
        if rsp_msg.MID != self.__req_message.MID or rsp_msg.RequestID != self.__req_message.RequestID:
            return False

        self.__rsp_msg_list.append(rsp_msg)
        if rsp_msg.IsLast is True:
            self.__rsp_last_msg = rsp_msg
            self.__Event.set()
        return True

    def wait_last_rsp(self, timeout: int):
        self.__Event.wait(timeout)
        self.__Event.clear()
        return self.__rsp_last_msg

    def get_rsp_last(self):
        return self.__rsp_last_msg

    def get_rsp_list(self):
        return self.__rsp_msg_list



from ...interface import IData
from ..market.ohlc_data import OHLCData
from ...packer.chart.ohlc_value_data_packer import OHLCValueDataPacker


class OHLCValueData(IData):
    def __init__(self, graph_id: str = '', ohlc_data: OHLCData = None):
        super().__init__(OHLCValueDataPacker(self))
        self._GraphID: str = graph_id
        self._OHLC: OHLCData = ohlc_data

    @property
    def GraphID(self):
        return self._GraphID

    @GraphID.setter
    def GraphID(self, value: str):
        self._GraphID = value

    @property
    def OHLC(self):
        return self._OHLC

    @OHLC.setter
    def OHLC(self, value: OHLCData):
        self._OHLC = value



import socket
import threading
import time
import zlib
import traceback

from .data.message_data import MessageData
from .interface import MsgID


class TSocket(object):
    __instance = None

    def __init__(self):
        self.__HOST = '127.0.0.1'
        self.__PORT = 12321
        self.__ADDR = (self.__HOST, self.__PORT)
        self.__TCP_SOCKET = None
        self.__RECV_T = None
        self.__RECV_BYTES = b''
        self.__BASE_CALLBACK = None
        self.__CHART_CALLBACK = None
        self.__MARKET_CALLBACK = None
        self.__TRADE_CALLBACK = None
        self.__SessionInited = False
        self.__IsRunning = False

    def __del__(self):
        self.__BASE_CALLBACK = None
        self.__CHART_CALLBACK = None
        self.__MARKET_CALLBACK = None
        self.__TRADE_CALLBACK = None
        self.close()

    def set_base_callback(self, base_callback):
        self.__BASE_CALLBACK = base_callback

    def set_chart_callback(self, chart_callback):
        self.__CHART_CALLBACK = chart_callback

    def set_market_callback(self, market_callback):
        self.__MARKET_CALLBACK = market_callback

    def set_trade_callback(self, trade_callback):
        self.__TRADE_CALLBACK = trade_callback

    def close(self):
        self.__IsRunning = False
        if self.__TCP_SOCKET is not None:
            self.__TCP_SOCKET.close()
        if self.__RECV_T is not None:
            self.__RECV_T.join()
        self.__RECV_T = None
        self.__TCP_SOCKET = None

    def is_connected(self):
        return self.__TCP_SOCKET is not None and self.__TCP_SOCKET._closed is False

    def connect(self, host: str = None, port: int = None, timeout: int = 30000):
        if self.is_connected() is True:
            return [True, '']

        if host is not None:
            self.__HOST = host
        if port is not None:
            self.__PORT = port
        self.__ADDR = (self.__HOST, self.__PORT)
        try:
            if self.__TCP_SOCKET is None:
                self.__TCP_SOCKET = socket.socket(family=socket.AF_INET, type=socket.SOCK_STREAM)
                self.__TCP_SOCKET.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                self.__TCP_SOCKET.setblocking(False)
            self.__TCP_SOCKET.settimeout(int(timeout / 1000))

            self.__TCP_SOCKET.connect(self.__ADDR)
        except socket.error as e:
            self.close()
            return [False, e.strerror]

        if self.__RECV_T is None:
            self.__IsRunning = True
            self.__RECV_T = threading.Thread(target=self.__recv_data)
            self.__RECV_T.start()

        while True:
            if self.__SessionInited is True:
                return [True, '']
            time.sleep(0.5)

        return [True, '']

    def send_message(self, msg: MessageData):
        if self.is_connected() is False:
            return False
        if len(msg.UData) > 102400:
            msg.CompressType = 0
            compressed = zlib.compress(msg.UData, -1)
            msg.UData = len(compressed).to_bytes(4, 'big') + compressed

        msgp = msg.pack()
        sendbytes = len(msgp).to_bytes(4, 'little') + msgp

        self.__TCP_SOCKET.sendall(sendbytes)
        return True

    def __recv_data(self):
        while self.__IsRunning:
            if self.is_connected() is False:
                break
            try:
                recv_data = self.__TCP_SOCKET.recv(1024)
                self.__RECV_BYTES += recv_data
            except Exception as e:
                self.__TCP_SOCKET.close()
                return [False, e]

            try:
                sz = int.from_bytes(self.__RECV_BYTES[:4], byteorder='little')
                while len(self.__RECV_BYTES) >= sz + 4:
                    msg_bytes = self.__RECV_BYTES[4:sz + 4]
                    self.__notify_msg(msg_bytes)
                    self.__RECV_BYTES = self.__RECV_BYTES[sz + 4:]
                    sz = int.from_bytes(self.__RECV_BYTES[:4], byteorder='little')
            except Exception as e:
                self.__TCP_SOCKET.close()
                traceback.print_exc()
                return [False, e]
            time.sleep(0.001)

    def __notify_msg(self, msg_bytes):
        msg = MessageData()
        if msg.un_pack(msg_bytes) is True:
            if msg.CompressType == 0:
                msg.UData = zlib.decompress(msg.UData)
            if msg.MID >= 1000 and msg.MID < 2000:
                if msg.MID == int(MsgID.MSGID_Base_SessionInit.value):
                    self.__SessionInited = True
                    return
                if self.__BASE_CALLBACK is not None:
                    self.__BASE_CALLBACK(msg)
            elif msg.MID >= 200000 and msg.MID < 300000:
                if self.__MARKET_CALLBACK is not None:
                    self.__MARKET_CALLBACK(msg)
            elif msg.MID >= 300000 and msg.MID < 400000:
                if self.__TRADE_CALLBACK is not None:
                    self.__TRADE_CALLBACK(msg)
            elif msg.MID >= 400000 and msg.MID < 500000:
                if self.__CHART_CALLBACK is not None:
                    self.__CHART_CALLBACK(msg)
        else:
            print('unpack errror')



from ...interface import IData
from .account_data import AccountData
from ...packer.trade.get_account_param_data_packer import GetAccountParamDataPacker


class GetAccountParamData(IData):
    def __init__(self, trade_name: str = ''):
        super().__init__(GetAccountParamDataPacker(self))
        self._TradeName = trade_name
        self._Account: AccountData = AccountData()

    @property
    def TradeName(self):
        return self._TradeName

    @TradeName.setter
    def TradeName(self, value: str):
        self._TradeName = value

    @property
    def Account(self):
        return self._Account

    @Account.setter
    def Account(self, value: AccountData):
        self._Account: AccountData = value



from ...interface import IPacker


class MarketParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.MarketNames)]

    def tuple_to_obj(self, t):
        if len(t) >= 1:
            self._obj.MarketNames = t[0]
            return True
        return False



from abc import abstractmethod
from .data.market.tick_data import TickData
from .data.market.ohlc_data import OHLCData
from .data.trade.order_data import OrderData
from .data.trade.tradeorder_data import TradeOrderData


class IListener(object):

    @abstractmethod
    def on_connect(self):
        pass

    @abstractmethod
    def on_disconnect(self):
        pass

    @abstractmethod
    def on_tick(self, tick: TickData):
        pass

    @abstractmethod
    def on_ohlc(self, ohlc: OHLCData):
        pass

    @abstractmethod
    def on_order_update(self, order: OrderData):
        pass

    @abstractmethod
    def on_tradeorder_update(self, tradeorder: TradeOrderData):
        pass
    



from ..interface import IData
from ..packer.login_data_packer import LoginDataPacker


class LoginData(IData):
    def __init__(self, user_id: str = '', password: str = '') -> None:
        super().__init__(LoginDataPacker(self))
        self._UserID: str = user_id
        self._PassWord: str = password

    @property
    def UserID(self):
        return self._UserID

    @UserID.setter
    def UserID(self, value: str):
        self._UserID = value

    @property
    def PassWord(self):
        return self._PassWord

    @PassWord.setter
    def PassWord(self, value: str):
        self._PassWord = value



from ...interface import IPacker


class FinancialGraphParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.ID), str(self._obj.Name), int(self._obj.Style), str(self._obj.AscColor),
                str(self._obj.AscBrush), str(self._obj.DescColor), str(self._obj.DescBrush), int(self._obj.PlotIndex),
                int(self._obj.ValueAxisID), float(self._obj.PriceTick), float(self._obj.TickValidMul), (self._obj.ShowLegend),
                str(self._obj.LegendFormat), str(self._obj.LegendColor), str(self._obj.BindInsID), str(self._obj.BindRange), bool(self._obj.JoinValueAxis),
                float(self._obj.ValidMaxValue), float(self._obj.ValidMinValue), self._obj.UserData, bool(self._obj.Visible)]

    def tuple_to_obj(self, t):
        if len(t) >= 21:
            self._obj.ID = t[0]
            self._obj.Name = t[1]
            self._obj.Style = t[2]
            self._obj.AscColor = t[3]
            self._obj.AscBrush = t[4]
            self._obj.DescColor = t[5]
            self._obj.DescBrush = t[6]
            self._obj.PlotIndex = t[7]
            self._obj.ValueAxisID = t[8]
            self._obj.PriceTick = t[9]
            self._obj.TickValidMul = t[10]
            self._obj.ShowLegend = t[11]
            self._obj.LegendFormat = t[12]
            self._obj.LegendColor = t[13]
            self._obj.BindInsID = t[14]
            self._obj.BindRange = t[15]
            self._obj.JoinValueAxis = t[16]
            self._obj.ValidMaxValue = t[17]
            self._obj.ValidMinValue = t[18]
            self._obj.UserData = t[19]
            self._obj.Visible = t[20]

            return True
        return False



from ...interface import IData
from ...packer.market.ohlc_data_packer import OHLCDataPacker


class OHLCData(IData):
    def __init__(self, exchange_id: str = '', instrument_id: str = '', trading_day: str = '',
                 trading_time: str = '', start_time: str = '', end_time: str = '', action_day: str = '',
                 action_time_span: int = -1, range: int = 60, index: int = -1, open_price: float = 0.0,
                 highest_price: float = 0.0, lowest_price: float = 0.0, close_price: float = 0.0, total_turnover: float = 0.0,
                 total_volume: int = 0, open_interest: float = 0.0, pre_settlement_price: float = 0.0, change_rate: float = 0.0,
                 change_value: float = 0.0, open_bid_price: float = 0.0, open_ask_price: float = 0.0, open_bid_volume: int = 0,
                 open_ask_volume: int = 0, highest_bid_price: float = 0.0, highest_ask_price: float = 0.0, highest_bid_volume: int = 0,
                 highest_ask_volume: int = 0, lowest_bid_price: float = 0.0, lowest_ask_price: float = 0.0, lowest_bid_volume: int = 0,
                 lowest_ask_volume: int = 0, close_bid_price: float = 0.0, close_ask_price: float = 0.0, close_bid_volume: int = 0,
                 close_ask_volume: int = 0):
        super().__init__(OHLCDataPacker(self))
        self._ExchangeID: str = exchange_id
        self._InstrumentID: str = instrument_id
        self._TradingDay: str = trading_day
        self._TradingTime: str = trading_time
        self._StartTime: str = start_time
        self._EndTime: str = end_time
        self._ActionDay: str = action_day
        self._ActionTimeSpan: int = action_time_span
        self._Range: int = range
        self._Index: int = index
        self._OpenPrice: float = open_price
        self._HighestPrice: float = highest_price
        self._LowestPrice: float = lowest_price
        self._ClosePrice: float = close_price
        self._TotalTurnover: float = total_turnover
        self._TotalVolume: int = total_volume
        self._OpenInterest: float = open_interest
        self._PreSettlementPrice: float = pre_settlement_price
        self._ChangeRate: float = change_rate
        self._ChangeValue: float = change_value
        self._OpenBidPrice: float = open_bid_price
        self._OpenAskPrice: float = open_ask_price
        self._OpenBidVolume: int = open_bid_volume
        self._OpenAskVolume: int = open_ask_volume
        self._HighestBidPrice: float = highest_bid_price
        self._HighestAskPrice: float = highest_ask_price
        self._HighestBidVolume: int = highest_bid_volume
        self._HighestAskVolume: int = highest_ask_volume
        self._LowestBidPrice: float = lowest_bid_price
        self._LowestAskPrice: float = lowest_ask_price
        self._LowestBidVolume: int = lowest_bid_volume
        self._LowestAskVolume: int = lowest_ask_volume
        self._CloseBidPrice: float = close_bid_price
        self._CloseAskPrice: float = close_ask_price
        self._CloseBidVolume: int = close_bid_volume
        self._CloseAskVolume: int = close_ask_volume

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def TradingDay(self):
        return self._TradingDay

    @TradingDay.setter
    def TradingDay(self, value: str):
        self._TradingDay = value

    @property
    def TradingTime(self):
        return self._TradingTime

    @TradingTime.setter
    def TradingTime(self, value: str):
        self._TradingTime = value

    @property
    def StartTime(self):
        return self._StartTime

    @StartTime.setter
    def StartTime(self, value: str):
        self._StartTime = value

    @property
    def EndTime(self):
        return self._EndTime

    @EndTime.setter
    def EndTime(self, value: str):
        self._EndTime = value

    @property
    def ActionDay(self):
        return self._ActionDay

    @ActionDay.setter
    def ActionDay(self, value: str):
        self._ActionDay = value

    @property
    def ActionTimeSpan(self):
        return self._ActionTimeSpan

    @ActionTimeSpan.setter
    def ActionTimeSpan(self, value: int):
        self._ActionTimeSpan = value

    @property
    def Range(self):
        return self._Range

    @Range.setter
    def Range(self, value: int):
        self._Range = value

    @property
    def Index(self):
        return self._Index

    @Index.setter
    def Index(self, value: int):
        self._Index = value

    @property
    def OpenPrice(self):
        return self._OpenPrice

    @OpenPrice.setter
    def OpenPrice(self, value: float):
        self._OpenPrice = value

    @property
    def HighestPrice(self):
        return self._HighestPrice

    @HighestPrice.setter
    def HighestPrice(self, value: float):
        self._HighestPrice = value

    @property
    def LowestPrice(self):
        return self._LowestPrice

    @LowestPrice.setter
    def LowestPrice(self, value: float):
        self._LowestPrice = value

    @property
    def ClosePrice(self):
        return self._ClosePrice

    @ClosePrice.setter
    def ClosePrice(self, value: float):
        self._ClosePrice = value

    @property
    def TotalTurnover(self):
        return self._TotalTurnover

    @TotalTurnover.setter
    def TotalTurnover(self, value: float):
        self._TotalTurnover = value

    @property
    def TotalVolume(self):
        return self._TotalVolume

    @TotalVolume.setter
    def TotalVolume(self, value: int):
        self._TotalVolume = value

    @property
    def OpenInterest(self):
        return self._OpenInterest

    @OpenInterest.setter
    def OpenInterest(self, value: float):
        self._OpenInterest = value

    @property
    def PreSettlementPrice(self):
        return self._PreSettlementPrice

    @PreSettlementPrice.setter
    def PreSettlementPrice(self, value: float):
        self._PreSettlementPrice = value

    @property
    def ChangeRate(self):
        return self._ChangeRate

    @ChangeRate.setter
    def ChangeRate(self, value: float):
        self._ChangeRate = value

    @property
    def ChangeValue(self):
        return self._ChangeValue

    @ChangeValue.setter
    def ChangeValue(self, value: float):
        self._ChangeValue = value

    @property
    def OpenBidPrice(self):
        return self._OpenBidPrice

    @OpenBidPrice.setter
    def OpenBidPrice(self, value: float):
        self._OpenBidPrice = value

    @property
    def OpenAskPrice(self):
        return self._OpenAskPrice

    @OpenAskPrice.setter
    def OpenAskPrice(self, value: float):
        self._OpenAskPrice = value

    @property
    def OpenBidVolume(self):
        return self._OpenBidVolume

    @OpenBidVolume.setter
    def OpenBidVolume(self, value: int):
        self._OpenBidVolume = value

    @property
    def OpenAskVolume(self):
        return self._OpenAskVolume

    @OpenAskVolume.setter
    def OpenAskVolume(self, value: int):
        self._OpenAskVolume = value

    @property
    def HighestBidPrice(self):
        return self._HighestBidPrice

    @HighestBidPrice.setter
    def HighestBidPrice(self, value: float):
        self._HighestBidPrice = value

    @property
    def HighestAskPrice(self):
        return self._HighestAskPrice

    @HighestAskPrice.setter
    def HighestAskPrice(self, value: float):
        self._HighestAskPrice = value

    @property
    def HighestBidVolume(self):
        return self._HighestBidVolume

    @HighestBidVolume.setter
    def HighestBidVolume(self, value: int):
        self._HighestBidVolume = value

    @property
    def HighestAskVolume(self):
        return self._HighestAskVolume

    @HighestAskVolume.setter
    def HighestAskVolume(self, value: int):
        self._HighestAskVolume = value

    @property
    def LowestBidPrice(self):
        return self._LowestBidPrice

    @LowestBidPrice.setter
    def LowestBidPrice(self, value: float):
        self._LowestBidPrice = value

    @property
    def LowestAskPrice(self):
        return self._LowestAskPrice

    @LowestAskPrice.setter
    def LowestAskPrice(self, value: float):
        self._LowestAskPrice = value

    @property
    def LowestBidVolume(self):
        return self._LowestBidVolume

    @LowestBidVolume.setter
    def LowestBidVolume(self, value: int):
        self._LowestBidVolume = value

    @property
    def LowestAskVolume(self):
        return self._LowestAskVolume

    @LowestAskVolume.setter
    def LowestAskVolume(self, value: int):
        self._LowestAskVolume = value

    @property
    def CloseBidPrice(self):
        return self._CloseBidPrice

    @CloseBidPrice.setter
    def CloseBidPrice(self, value: float):
        self._CloseBidPrice = value

    @property
    def CloseAskPrice(self):
        return self._CloseAskPrice

    @CloseAskPrice.setter
    def CloseAskPrice(self, value: float):
        self._CloseAskPrice = value

    @property
    def CloseBidVolume(self):
        return self._CloseBidVolume

    @CloseBidVolume.setter
    def CloseBidVolume(self, value: int):
        self._CloseBidVolume = value

    @property
    def CloseAskVolume(self):
        return self._CloseAskVolume

    @CloseAskVolume.setter
    def CloseAskVolume(self, value: int):
        self._CloseAskVolume = value



from typing import List
from ...interface import IData
from ...packer.trade.read_history_tradeorder_param_data_packer import ReadHistoryTradeOrderParamDataPacker
from .tradeorder_data import TradeOrderData


class ReadHistoryTradeOrderParamData(IData):
    def __init__(self, start_date: str = '', end_date: str = '', data_list: List[TradeOrderData] = []):
        super().__init__(ReadHistoryTradeOrderParamDataPacker(self))
        self._StartDate: str = start_date
        self._EndDate: str = end_date
        self._DataList: List[TradeOrderData] = data_list

    @property
    def StartDate(self):
        return self._StartDate

    @StartDate.setter
    def StartDate(self, value: str):
        self._StartDate = value

    @property
    def EndDate(self):
        return self._EndDate

    @EndDate.setter
    def EndDate(self, value: str):
        self._EndDate = value

    @property
    def DataList(self):
        return self._DataList

    @DataList.setter
    def DataList(self, value: List[TradeOrderData]):
        self._DataList = value



from ...interface import IData
from ...packer.market.sub_ohlc_param_data_packer import SubOHLCParamDataPacker


class SubOHLCParamData(IData):
    def __init__(self, market_name: str = '', exchange_id: str = '', instrument_id: str = '', range: str = "60"):
        super().__init__(SubOHLCParamDataPacker(self))
        self._MarketName: str = market_name
        self._ExchangeID: str = exchange_id
        self._InstrumentID: str = instrument_id
        self._Range: str = range

    @property
    def MarketName(self):
        return self._MarketName

    @MarketName.setter
    def MarketName(self, value: str):
        self._MarketName = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def Range(self):
        return self._Range

    @Range.setter
    def Range(self, value: str):
        self._Range = value



from typing import Dict
from ...interface import IData
from ...packer.chart.bar_graph_param_data_packer import BarGraphParamDataPacker


class BarGraphParamData(IData):

    def __init__(self, id: str = '', name: str = '', style: int = 0, frame_style: int = 0, color: str = "white",
                 price_tick: float = 1.0, tick_valid_mul: float = -1.0, line_width: int = 1, plot_index: int = 0,
                 value_axis_id: int = -1, show_legend: bool = True, legend_format: str = '', legend_color: str = "",
                 join_value_axis: bool = True, valid_max_value: float = 9999999.99, valid_min_value: float = -9999999.99,
                 user_data: Dict[str, str] = {}, visible: bool = True):
        """柱状图配置参数

        Args:
            id (str, optional):  图唯一编码，不能重复. Defaults to ''.
            name (str, optional): 图名称. Defaults to ''.
            style (int, optional): 样式 -1 NoStyle, 0 Box,1000 UserStyle. Defaults to 0.
            frame_style (int, optional): 边框样式 0 无边框 1 线形 2 3D效果. Defaults to 0.
            color (str, optional): 颜色 white #FFFFFF 格式. Defaults to "white".
            price_tick (float, optional): 有效的变动值. Defaults to 1.0.
            tick_valid_mul (float, optional): 有效倍数,超过倍数的不会计算在轴显示范围内. Defaults to -1.0.
            line_width (int, optional): . Defaults to 1.
            plot_index (int, optional): 图形所属图块. Defaults to 0.
            value_axis_id (int, optional): 图形所属y轴 -1左侧第一个Y轴 -2 左侧第二个,以此类推。1右侧第一个Y轴. Defaults to -1.
            show_legend (bool, optional): 是否显示在legend上. Defaults to True.
            legend_format (str, optional): 在legend上显示的格式,默认是name:value. Defaults to ''.
            legend_color (str, optional): legend上显示的颜色,默认与图的颜色相同. Defaults to "".
            join_value_axis (bool, optional): 是否加入到Y轴显示范围的计算. Defaults to True.
            valid_max_value (float, optional): 最大有效值. Defaults to 9999999.99.
            valid_min_value (float, optional): 最小有效值. Defaults to -9999999.99.
            user_data (Dict[str, str], optional): 用户自定义数据. Defaults to {}.
            visible (bool, optional): 是否显示 Defaults to True.
        """
        super().__init__(BarGraphParamDataPacker(self))
        self._ID: str = id
        self._Name: str = name
        self._Style: int = style
        self._FrameStyle: int = frame_style
        self._Color: str = color
        self._PriceTick: float = price_tick
        self._LineWidth: int = line_width
        self._PlotIndex: int = plot_index
        self._ValueAxisID: int = value_axis_id
        self._ShowLegend: bool = show_legend
        self._LegendFormat: str = legend_format
        self._LegendColor: str = legend_color
        self._JoinValueAxis: bool = join_value_axis
        self._TickValidMul: float = tick_valid_mul
        self._ValidMaxValue: float = valid_max_value
        self._ValidMinValue: float = valid_min_value
        self._UserData: Dict[str, str] = user_data
        self._Visible: bool = visible

    @property
    def ID(self):
        return self._ID

    @ID.setter
    def ID(self, value: str):
        self._ID = value

    @property
    def Name(self):
        return self._Name

    @Name.setter
    def Name(self, value: str):
        self._Name = value

    @property
    def Style(self):
        return self._Style

    @Style.setter
    def Style(self, value: int):
        self._Style = value

    @property
    def FrameStyle(self):
        return self._FrameStyle

    @FrameStyle.setter
    def FrameStyle(self, value: int):
        self._FrameStyle = value

    @property
    def Color(self):
        return self._Color

    @Color.setter
    def Color(self, value: str):
        self._Color = value

    @property
    def PriceTick(self):
        return self._PriceTick

    @PriceTick.setter
    def PriceTick(self, value):
        self._PriceTick = float(value)

    @property
    def LineWidth(self):
        return self._LineWidth

    @LineWidth.setter
    def LineWidth(self, value: int):
        self._LineWidth = value

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value: int):
        self._PlotIndex = value

    @property
    def ValueAxisID(self):
        return self._ValueAxisID

    @ValueAxisID.setter
    def ValueAxisID(self, value: int):
        self._ValueAxisID = value

    @property
    def ShowLegend(self):
        return self._ShowLegend

    @ShowLegend.setter
    def ShowLegend(self, value: bool):
        self._ShowLegend = value

    @property
    def LegendFormat(self):
        return self._LegendFormat

    @LegendFormat.setter
    def LegendFormat(self, value: str):
        self._LegendFormat = value

    @property
    def LegendColor(self):
        return self._LegendColor

    @LegendColor.setter
    def LegendColor(self, value: str):
        self._LegendColor = value

    @property
    def JoinValueAxis(self):
        return self._JoinValueAxis

    @JoinValueAxis.setter
    def JoinValueAxis(self, value: bool):
        self._JoinValueAxis = value

    @property
    def ValidMaxValue(self):
        return self._ValidMaxValue

    @ValidMaxValue.setter
    def ValidMaxValue(self, value):
        self._ValidMaxValue = float(value)

    @property
    def ValidMinValue(self):
        return self._ValidMinValue

    @ValidMinValue.setter
    def ValidMinValue(self, value):
        self._ValidMinValue = float(value)

    @property
    def TickValidMul(self):
        return self._TickValidMul

    @TickValidMul.setter
    def TickValidMul(self, value):
        self._TickValidMul = float(value)

    @property
    def UserData(self):
        return self._UserData

    @UserData.setter
    def UserData(self, value: Dict[str, str]):
        self._UserData = value

    @property
    def Visible(self):
        return self._Visible

    @JoinValueAxis.setter
    def Visible(self, value: bool):
        self._Visible = value



from typing import List
from ...interface import IData
from .position_data import PositionData
from ...packer.trade.get_positions_param_data_packer import GetPositionsParamDataPacker


class GetPositionsParamData(IData):
    def __init__(self, trade_name: str = '', exchange_id: str = '', instrument_id: str = ''):
        super().__init__(GetPositionsParamDataPacker(self))
        self._TradeName = trade_name
        self._ExchangeID = exchange_id
        self._InstrumentID = instrument_id
        self._DataList: List[PositionData] = []

    @property
    def TradeName(self):
        return self._TradeName

    @TradeName.setter
    def TradeName(self, value: str):
        self._TradeName = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def DataList(self):
        return self._DataList

    @DataList.setter
    def DataList(self, value: List[PositionData]):
        self._DataList: List[PositionData] = value



from typing import Tuple

from .req_rsp import ReqRspDict, ReqRsp
from ..listener import IListener
from ..interface import IData, MsgID
from ..tsocket import TSocket
from ..data.message_data import MessageData
from ..data.market.tick_data import TickData
from ..data.market.ohlc_data import OHLCData
from ..data.market.market_param_data import MarketParamData
from ..data.market.history_ohlc_param_data import HistoryOHLCParamData
from ..data.market.sub_ohlc_param_data import SubOHLCParamData
from ..data.market.query_param_data import QueryParamData
from ..data.market.save_ohlc_list_param_data import SaveOHLCListParamData
from ..data.market.save_tick_list_param_data import SaveTickListParamData
import pandas as pd


class MarketHandle():
    __ReqID: int = 0
    __Listener: IListener = None
    __ReqRspDict: ReqRspDict = ReqRspDict()

    def __init__(self, tsocket: TSocket):
        self.__TSocket = tsocket
        self.__TSocket.set_market_callback(self.__recv_msg)

    def set_callback(self, **kwargs):
        if kwargs is None:
            return
        for key in kwargs:
            setattr(self, key, kwargs[key])

    def set_listener(self, listener: IListener):
        self.__Listener = listener

    def set_market_params(self, params: MarketParamData) -> Tuple[bool, str]:
        return self.__wait_send_msg(int(MsgID.MSGID_Market_SetParams.value), params)

    def subscribe(self, params: QueryParamData) -> Tuple[bool, str]:
        return self.__wait_send_msg(int(MsgID.MSGID_Market_Sub.value), params)

    def subscribe_ohlc(self, params: SubOHLCParamData) -> Tuple[bool, str]:
        return self.__wait_send_msg(int(MsgID.MSGID_Market_SubOHLC.value), params)

    def save_ohlc_list(self, params: SaveOHLCListParamData) -> Tuple[bool, str]:
        return self.__wait_send_msg(int(MsgID.MSGID_Market_SaveOHLCList.value), params)

    def save_tick_list(self, params: SaveTickListParamData) -> Tuple[bool, str]:
        return self.__wait_send_msg(int(MsgID.MSGID_Market_SaveTickList.value), params)

    def get_history_ohlc(self, params: HistoryOHLCParamData) -> Tuple[bool, str, pd.DataFrame]:
        self.__ReqID = self.__ReqID + 1
        mid = int(MsgID.MSGID_Market_GetHistoryOHLC.value)
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            msg.UData = params.pack()

        key = '%s_%s' % (mid, self.__ReqID)
        req_rsp = self.__ReqRspDict.new_reqrsp(key, msg)
        if self.__TSocket.send_message(msg) is False:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令失败', None]

        rsp = req_rsp.wait_last_rsp(60)
        if rsp is None:
            self.__ReqRspDict.remove(key)
            return [False, '获取历史OHLC数据超时', None]

        ret = [True, "", None]
        if params.IsReturnList is True:
            ret = [rsp.RspSuccess, rsp.RspMsg, self.__unpack_ohlc_list(req_rsp)]
        else:
            ret = [rsp.RspSuccess, rsp.RspMsg, self.__unpack_ohlc_dataframe(req_rsp)]

        self.__ReqRspDict.remove(key)
        return ret

    def __notify_on_tick(self, msg: MessageData):
        hasontick = hasattr(self, 'on_tick')
        if hasontick is False and self.__Listener is None:
            print('未定义任何on_tick回调方法')
            return
        t = TickData()
        if t.un_pack(msg.UData) is True:
            if hasontick is True:
                self.on_tick(t)
            if self.__Listener is not None:
                self.__Listener.on_tick(t)

    def __notify_on_ohlc(self, msg: MessageData):
        hasonohlc = hasattr(self, 'on_ohlc')
        if hasonohlc is False and self.__Listener is None:
            print('未定义任何on_ohlc回调方法')
            return
        o = OHLCData()
        if o.un_pack(msg.UData) is True:
            if hasonohlc is True:
                self.on_ohlc(o)
            if self.__Listener is not None:
                self.__Listener.on_ohlc(o)

    def __unpack_ohlc_list(self, reqrsp: ReqRsp):
        ohlcs = list()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = HistoryOHLCParamData()
                rspparams.un_pack(r.UData)
                for ot in rspparams.OHLCList:
                    o = OHLCData()
                    o.tuple_to_obj(ot)
                    ohlcs.append(o)
        return ohlcs

    def __unpack_ohlc_dataframe(self, reqrsp: ReqRsp):
        dfrtn = pd.DataFrame()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = HistoryOHLCParamData()
                rspparams.un_pack(r.UData)
                df = pd.DataFrame(rspparams.OHLCList, columns=['ExchangeID', 'InstrumentID', 'TradingDay', 'TradingTime', 'StartTime', 'EndTime', 'ActionDay',
                                                               'ActionTimeSpan', 'Range', 'Index', 'OpenPrice', 'HighestPrice', 'LowestPrice', 'ClosePrice',
                                                               'TotalTurnover', 'TotalVolume', 'OpenInterest', 'PreSettlementPrice', 'ChangeRate', 'ChangeValue',
                                                               'OpenBidPrice', 'OpenAskPrice', 'OpenBidVolume', 'OpenAskVolume', 'HighestBidPrice', 'HighestAskPrice',
                                                               'HighestBidVolume', 'HighestAskVolume', 'LowestBidPrice', 'LowestAskPrice', 'LowestBidVolume', 'LowestAskVolume',
                                                               'CloseBidPrice', 'CloseAskPrice', 'CloseBidVolume', 'CloseAskVolume'])
                dfrtn = pd.concat([dfrtn, df], ignore_index=True, copy=False)
        return dfrtn

    def __recv_msg(self, msg: MessageData):
        if msg.MID == int(MsgID.MSGID_Market_Tick.value):
            self.__notify_on_tick(msg)
            return
        elif msg.MID == int(MsgID.MSGID_Market_OHLC.value):
            self.__notify_on_ohlc(msg)
            return

        key = '%s_%s' % (msg.MID, msg.RequestID)
        reqrsp: ReqRsp = self.__ReqRspDict.get_reqrsp(key)
        if reqrsp is not None:
            reqrsp.append_rsp(msg)

    def __wait_send_msg(self, mid, params: IData):
        self.__ReqID = self.__ReqID + 1
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            msg.UData = params.pack()

        key = '%s_%s' % (mid, self.__ReqID)

        req_rsp = self.__ReqRspDict.new_reqrsp(key, msg)
        if self.__TSocket.send_message(msg) is False:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令失败']

        rsp = req_rsp.wait_last_rsp(10)
        if rsp is None:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令超时']

        ret = [rsp.RspSuccess, rsp.RspMsg]
        self.__ReqRspDict.remove(key)
        return ret



from typing import List
from ...interface import IData
from ...packer.market.save_tick_list_param_data_packer import SaveTickListParamDataPacker


class SaveTickListParamData(IData):
    _PreClosePrice: float = -1.0

    def __init__(self,
                 market_name: str = '',
                 exchange_id: str = '',
                 instrument_id: str = '',
                 product_id: str = '',
                 trading_day: str = '',
                 pre_close_price: float = -1.0,
                 pre_settlement_price: float = -1.0,
                 pre_open_interest: float = -1.0,
                 upper_limit_price: float = -1.0,
                 lower_limit_price: float = -1.0,
                 action_day_list: List[str] = [],
                 trading_time_list: List[str] = [],
                 update_mill_sec_list: List[int] = [],
                 local_time_list: List[int] = [],
                 last_price_list: List[float] = [],
                 last_volume_list: List[int] = [],
                 bid_price_list: List[float] = [],
                 bid_volume_list: List[int] = [],
                 ask_price_list: List[float] = [],
                 ask_volume_list: List[int] = [],
                 avg_price_list: List[float] = [],
                 open_price_list: List[float] = [],
                 high_price_list: List[float] = [],
                 lower_price_list: List[float] = [],
                 total_turnover_list: List[float] = [],
                 total_volume_list: List[int] = [],
                 open_interest_list: List[float] = [],
                 close_price_list: List[float] = [],
                 settlement_price_list: List[float] = [],
                 total_value_list: List[float] = []
                 ):
        super().__init__(SaveTickListParamDataPacker(self))
        self._MarketName: str = market_name
        self._ExchangeID: str = exchange_id
        self._InstrumentID: str = instrument_id
        self._ProductID: str = product_id
        self._TradingDay: str = trading_day
        self._PreClosePrice: float = float(pre_close_price)
        self._PreSettlementPrice: float = pre_settlement_price
        self._PreOpenInterest: float = pre_open_interest
        self._UpperLimitPrice: float = upper_limit_price
        self._LowerLimitPrice: float = lower_limit_price
        self._ActionDayList: List[str] = action_day_list
        self._TradingTimeList: List[str] = trading_time_list
        self._UpdateMillSecList: List[int] = update_mill_sec_list
        self._LocalTimeList: List[int] = local_time_list
        self._LastPriceList: List[float] = last_price_list
        self._LastVolumeList: List[int] = last_volume_list
        self._BidPriceList: List[float] = bid_price_list
        self._BidVolumeList: List[int] = bid_volume_list
        self._AskPriceList: List[float] = ask_price_list
        self._AskVolumeList: List[int] = ask_volume_list
        self._AvgPriceList: List[float] = avg_price_list
        self._OpenPriceList: List[float] = open_price_list
        self._HighPriceList: List[float] = high_price_list
        self._LowerPriceList: List[float] = lower_price_list
        self._TotalTurnoverList: List[float] = total_turnover_list
        self._TotalVolumeList: List[int] = total_volume_list
        self._OpenInterestList: List[float] = open_interest_list
        self._ClosePriceList: List[float] = close_price_list
        self._SettlementPriceList: List[float] = settlement_price_list
        self._TotalValueList: List[float] = total_value_list

    @property
    def MarketName(self):
        return self._MarketName

    @MarketName.setter
    def MarketName(self, value: str):
        self._MarketName = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def ProductID(self):
        return self._ProductID

    @ProductID.setter
    def ProductID(self, value: str):
        self._ProductID = value

    @property
    def TradingDay(self):
        return self._TradingDay

    @TradingDay.setter
    def TradingDay(self, value: str):
        self._TradingDay = value

    @property
    def PreClosePrice(self):
        return self._PreClosePrice

    @PreClosePrice.setter
    def PreClosePrice(self, value: float):
        self._PreClosePrice = value

    @property
    def PreSettlementPrice(self):
        return self._PreSettlementPrice

    @PreSettlementPrice.setter
    def PreSettlementPrice(self, value: float):
        self._PreSettlementPrice = value

    @property
    def PreOpenInterest(self):
        return self._PreOpenInterest

    @PreOpenInterest.setter
    def PreOpenInterest(self, value: float):
        self._PreOpenInterest = value

    @property
    def UpperLimitPrice(self):
        return self._UpperLimitPrice

    @UpperLimitPrice.setter
    def UpperLimitPrice(self, value: float):
        self._UpperLimitPrice = value

    @property
    def LowerLimitPrice(self):
        return self._LowerLimitPrice

    @LowerLimitPrice.setter
    def LowerLimitPrice(self, value: float):
        self._LowerLimitPrice = value

    @property
    def ActionDayList(self):
        return self._ActionDayList

    @ActionDayList.setter
    def ActionDay(self, value: List[str]):
        self._ActionDayList = value

    @property
    def TradingTimeList(self):
        return self._TradingTimeList

    @TradingTimeList.setter
    def TradingTimeList(self, value: List[str]):
        self._TradingTimeList = value

    @property
    def UpdateMillSecList(self):
        return self._UpdateMillSecList

    @UpdateMillSecList.setter
    def UpdateMillSecList(self, value: List[int]):
        self._UpdateMillSecList = value

    @property
    def LocalTimeList(self):
        return self._LocalTimeList

    @LocalTimeList.setter
    def LocalTimeList(self, value: List[int]):
        self._LocalTimeList = value

    @property
    def LastPriceList(self):
        return self._LastPriceList

    @LastPriceList.setter
    def LastPriceList(self, value: List[float]):
        self._LastPriceList = value

    @property
    def LastVolumeList(self):
        return self._LastVolumeList

    @LastVolumeList.setter
    def LastVolumeList(self, value: List[int]):
        self._LastVolumeList = value

    @property
    def BidPriceList(self):
        return self._BidPriceList

    @BidPriceList.setter
    def BidPriceList(self, value: List[float]):
        self._BidPriceList = value

    @property
    def BidVolumeList(self):
        return self._BidVolumeList

    @BidVolumeList.setter
    def BidVolumeList(self, value: List[int]):
        self._BidVolumeList = value

    @property
    def AskPriceList(self):
        return self._AskPriceList

    @AskPriceList.setter
    def AskPriceList(self, value: List[float]):
        self._AskPriceList = value

    @property
    def AskVolumeList(self):
        return self._AskVolumeList

    @AskVolumeList.setter
    def AskVolumeList(self, value: List[int]):
        self._AskVolumeList = value

    @property
    def AvgPriceList(self):
        return self._AvgPriceList

    @AvgPriceList.setter
    def AvgPriceList(self, value: List[float]):
        self._AvgPriceList = value

    @property
    def OpenPriceList(self):
        return self._OpenPriceList

    @OpenPriceList.setter
    def OpenPriceList(self, value: List[float]):
        self._OpenPriceList = value

    @property
    def HighPriceList(self):
        return self._HighPriceList

    @HighPriceList.setter
    def HighPriceList(self, value: List[float]):
        self._HighPriceList = value

    @property
    def LowerPriceList(self):
        return self._LowerPriceList

    @LowerPriceList.setter
    def LowerPriceList(self, value: List[float]):
        self._LowerPriceList = value

    @property
    def TotalTurnoverList(self):
        return self._TotalTurnoverList

    @TotalTurnoverList.setter
    def TotalTurnoverList(self, value: List[float]):
        self._TotalTurnoverList = value

    @property
    def TotalVolumeList(self):
        return self._TotalVolumeList

    @TotalVolumeList.setter
    def TotalVolumeList(self, value: List[int]):
        self._TotalVolumeList = value

    @property
    def OpenInterestList(self):
        return self._OpenInterestList

    @OpenInterestList.setter
    def OpenInterestList(self, value: List[float]):
        self._OpenInterestList = value

    @property
    def ClosePriceList(self):
        return self._ClosePriceList

    @ClosePriceList.setter
    def ClosePriceList(self, value: List[float]):
        self._ClosePriceList = value

    @property
    def SettlementPriceList(self):
        return self._SettlementPriceList

    @SettlementPriceList.setter
    def SettlementPriceList(self, value: List[float]):
        self._SettlementPriceList = value

    @property
    def TotalValueList(self):
        return self._TotalValueList

    @TotalValueList.setter
    def TotalValueList(self, value: List[float]):
        self._TotalValueList = value



from .req_rsp import ReqRspDict, ReqRsp
from ..interface import IData, MsgID
from ..tsocket import TSocket
from ..data.message_data import MessageData
from ..listener import IListener
from ..data.trade.order_data import OrderData
from ..data.trade.tradeorder_data import TradeOrderData
from ..data.trade.read_history_order_param_data import ReadHistoryOrderParamData
from ..data.trade.read_history_tradeorder_param_data import ReadHistoryTradeOrderParamData
from ..data.trade.trade_param_data import TradeParamData
from ..data.trade.get_account_param_data import GetAccountParamData
from ..data.trade.get_orders_param_data import GetOrdersParamData
from ..data.trade.get_tradeorders_param_data import GetTradeOrdersParamData
from ..data.trade.get_positions_param_data import GetPositionsParamData


class TradeHandle():
    __ReqID: int = 0
    __Listener: IListener = None
    __ReqRspDict: ReqRspDict = ReqRspDict()

    def __init__(self, tsocket: TSocket):
        self.__TSocket = tsocket
        self.__TSocket.set_trade_callback(self.__recv_msg)

    def __del__(self):
        self.__TSocket.set_trade_callback(None)

    def set_callback(self, **kwargs):
        if kwargs is None:
            return
        for key in kwargs:
            setattr(self, key, kwargs[key])

    def set_listener(self, listener: IListener):
        self.__Listener = listener

    def set_trade_params(self, params: TradeParamData):
        return self.__wait_send_msg(int(MsgID.Trade_SetParams.value), params)

    def insert_order(self, order: OrderData):
        return self.__wait_send_msg(int(MsgID.Trade_InsertOrder.value), order)

    def cancel_order(self, order: OrderData):
        return self.__wait_send_msg(int(MsgID.Trade_CancelOrder.value), order)

    def get_orders(self, params: GetOrdersParamData):
        return self.wait_read_datas(int(MsgID.Trade_GetOrders.value), params, '读取委托单', self.__unpack_get_order_list)

    def get_tradeorders(self, params: GetTradeOrdersParamData):
        return self.wait_read_datas(int(MsgID.Trade_GetTradeOrders.value), params, '读取成交单', self.__unpack_get_tradeorder_list)

    def get_positions(self, params: GetPositionsParamData):
        return self.wait_read_datas(int(MsgID.Trade_GetPositions.value), params, '读取持仓单', self.__unpack_position_list)

    def get_account(self, params: GetAccountParamData):
        return self.wait_read_datas(int(MsgID.Trade_GetAccount.value), params, '读取账户', self.__unpack_account_list)

    def read_history_orders(self, params: ReadHistoryOrderParamData):
        return self.wait_read_datas(int(MsgID.Trade_ReadHistoryOrders.value), params, '读取历史委托单', self.__unpack_order_list)

    def read_history_tradeorders(self, params: ReadHistoryTradeOrderParamData):
        return self.wait_read_datas(int(MsgID.Trade_ReadHistoryTradeOrders.value), params, '读取历史成交单', self.__unpack_tradeorder_list)

    def __notify_order_update(self, msg: MessageData):
        hason = hasattr(self, 'on_order_update')
        if hason is False and self.__Listener is None:
            print('未定义任何on_order_update回调方法')
            return
        order = OrderData()
        if order.un_pack(msg.UData) is True:
            if hason is True:
                self.on_order_update(order)
            if self.__Listener is not None:
                self.__Listener.on_order_update(order)

    def __notify_tradeorder_update(self, msg: MessageData):
        hason = hasattr(self, 'on_tradeorder_update')
        if hason is False and self.__Listener is None:
            print('未定义任何on_tradeorder_update回调方法')
            return
        tradeorder = TradeOrderData()
        if tradeorder.un_pack(msg.UData) is True:
            if hason is True:
                self.on_tradeorder_update(tradeorder)
            if self.__Listener is not None:
                self.__Listener.on_tradeorder_update(tradeorder)

    def __unpack_get_order_list(self, reqrsp: ReqRsp):
        os = list()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = GetOrdersParamData()
                if rspparams.un_pack(r.UData) is False:
                    continue
                for ot in rspparams.DataList:
                    os.append(ot)
        return os

    def __unpack_get_tradeorder_list(self, reqrsp: ReqRsp):
        os = list()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = GetTradeOrdersParamData()
                if rspparams.un_pack(r.UData) is False:
                    continue
                for ot in rspparams.DataList:
                    os.append(ot)
        return os

    def __unpack_order_list(self, reqrsp: ReqRsp):
        os = list()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = ReadHistoryOrderParamData()
                if rspparams.un_pack(r.UData) is False:
                    continue
                for ot in rspparams.DataList:
                    os.append(ot)
        return os

    def __unpack_tradeorder_list(self, reqrsp: ReqRsp):
        os = list()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = ReadHistoryTradeOrderParamData()
                rspparams.un_pack(r.UData)
                for ot in rspparams.DataList:
                    os.append(ot)
        return os

    def __unpack_position_list(self, reqrsp: ReqRsp):
        ps = list()
        rsp_list = reqrsp.get_rsp_list()
        for r in rsp_list:
            if len(r.UData) > 0:
                rspparams = GetPositionsParamData()
                rspparams.un_pack(r.UData)
                for ot in rspparams.DataList:
                    ps.append(ot)
        return ps

    def __unpack_account_list(self, reqrsp: ReqRsp):
        rsp_last = reqrsp.get_rsp_last()
        if len(rsp_last.UData) > 0:
            acc = GetAccountParamData()
            acc.un_pack(rsp_last.UData)
            return acc.Account

        return None

    def wait_read_datas(self, mid, params: IData, fun_name: str, unpack_func):
        self.__ReqID = self.__ReqID + 1
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            msg.UData = params.pack()

        key = '%s_%s' % (mid, self.__ReqID)
        req_rsp = self.__ReqRspDict.new_reqrsp(key, msg)
        if self.__TSocket.send_message(msg) is False:
            self.__ReqRspDict.remove(key)
            return [False, '发送%s失败' % fun_name, None]

        rsp = req_rsp.wait_last_rsp(10)
        if rsp is None:
            self.__ReqRspDict.remove(key)
            return [False, '获取%s数据超时' % fun_name, None]

        return [True, '获取%s数据成功' % fun_name, unpack_func(req_rsp)]

    def __recv_msg(self, msg: MessageData):
        if msg.MID == int(MsgID.Trade_OrderUpdate.value):
            self.__notify_order_update(msg)
            return

        elif msg.MID == int(MsgID.Trade_TradeOrderUpdate.value):
            self.__notify_tradeorder_update(msg)
            return

        key = '%s_%s' % (msg.MID, msg.RequestID)
        reqrsp: ReqRsp = self.__ReqRspDict.get_reqrsp(key)
        if reqrsp is not None:
            reqrsp.append_rsp(msg)

    def __wait_send_msg(self, mid, params: IData):
        self.__ReqID = self.__ReqID + 1
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            msg.UData = params.pack()

        key = '%s_%s' % (mid, self.__ReqID)

        req_rsp = self.__ReqRspDict.new_reqrsp(key, msg)
        if self.__TSocket.send_message(msg) is False:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令失败']

        rsp = req_rsp.wait_last_rsp(10)
        if rsp is None:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令超时']
        return [rsp.RspSuccess, rsp.RspMsg]



import threading
from .tsocket import TSocket
from .listener import IListener

from .handle.chart_handle import ChartHandle
from .handle.base_handle import BaseHandle
from .handle.trade_handle import TradeHandle
from .handle.market_handle import MarketHandle


class FinClient(object):
    __TSocket = None
    __ChartHandle = None
    __BaseHandle = None
    __MarketHandle = None
    __TradeHandle = None
    __instance_lock = threading.Lock()

    def __init__(self):
        self.__TSocket = TSocket()
        self.__ChartHandle = ChartHandle(self.__TSocket)
        self.__BaseHandle = BaseHandle(self.__TSocket)
        self.__MarketHandle = MarketHandle(self.__TSocket)
        self.__TradeHandle = TradeHandle(self.__TSocket)

    def __del__(self):
        self.__TSocket.close()
        del self.__TradeHandle
        del self.__MarketHandle
        del self.__ChartHandle
        del self.__BaseHandle
        del self.__TSocket

    @classmethod
    def instance(cls):
        if not hasattr(cls, "_instance"):
            with cls.__instance_lock:
                if not hasattr(cls, "_instance"):
                    cls._instance = FinClient()
        return cls._instance

    def set_callback(self, **kwargs):
        if kwargs is None:
            return
        for key in kwargs:
            if key == "on_tick":
                self.__MarketHandle.set_callback(on_tick=kwargs[key])
            elif key == "on_ohlc":
                self.__MarketHandle.set_callback(on_ohlc=kwargs[key])
            elif key == "on_order_update":
                self.__TradeHandle.set_callback(on_order_update=kwargs[key])
            elif key == "on_tradeorder_update":
                self.__TradeHandle.set_callback(on_tradeorder_update=kwargs[key])

    def set_listener(self, listener: IListener):
        self.__MarketHandle.set_listener(listener)
        self.__TradeHandle.set_listener(listener)

    def base_handle(self):
        return self.__BaseHandle

    def chart_handle(self):
        return self.__ChartHandle

    def market_handle(self):
        return self.__MarketHandle

    def trade_handle(self):
        return self.__TradeHandle

    def connect(self, host: str = None, port: int = None, timeout: int = 30000):
        return self.__TSocket.connect(host, port, timeout)

    def close(self):
        self.__TSocket.close()



from abc import abstractmethod
from enum import Enum
import msgpack


class MsgID(Enum):
    MSGID_Base_Heart = 1000
    MSGID_Base_SessionInit = 1001
    MSGID_Base_Login = 1002

    MSGID_Market_SetParams = 200001
    MSGID_Market_Open = 200002
    MSGID_Market_Close = 200003
    MSGID_Market_Sub = 200004
    MSGID_Market_UnSub = 200005
    MSGID_Market_Tick = 200006
    MSGID_Market_OHLC = 200007
    MSGID_Market_SubOHLC = 200008
    MSGID_Market_UnSubOHLC = 200009
    MSGID_Market_GetHistoryOHLC = 200010
    MSGID_Market_SaveOHLC = 200011
    MSGID_Market_SaveOHLCList = 200012
    MSGID_Market_SaveTickList = 200013

    Trade_SetParams = 300010
    Trade_Open = 300011
    Trade_Close = 300012
    Trade_Inited = 300013
    Trade_InsertOrder = 300021
    Trade_CancelOrder = 300022
    Trade_OrderUpdate = 300023
    Trade_TradeOrderUpdate = 300024
    Trade_QueryOrders = 300025
    Trade_QueryTradeOrders = 300026
    Trade_ReadHistoryOrders = 300100
    Trade_ReadHistoryTradeOrders = 300101
    Trade_GetOrders = 300201
    Trade_GetTradeOrders = 300202
    Trade_GetPositions = 300203
    Trade_GetAccount = 300204

    Chart_Set_Init_Param = 400000
    Chart_Set_Chart_Param = 400001
    Chart_Init_Show = 400002
    Chart_AddLineGraph = 400011
    Chart_AddFinancialGraph = 400012
    Chart_AddTextGraph = 400013
    Chart_AddMarkerGraph = 400014
    Chart_AddBarGraph = 400015
    Chart_AddGraphValue = 400026
    Chart_AddOHLCValue = 400027
    Chart_AddOHLCValueList = 400028
    Chart_AddGraphValueList = 400029
    Chart_AddTimeSpanGVList = 400030
    Chart_SaveChartData = 400031
    Chart_LoadChartData = 400032


class IPacker(object):
    def __init__(self, obj) -> None:
        super().__init__()
        self._obj = obj
        if self._obj is None:
            raise Exception("packer data is none")

    def pack(self):
        return msgpack.packb(self.obj_to_tuple())

    @abstractmethod
    def obj_to_tuple(self):
        return False

    def un_pack(self, data_bytes):
        try:
            t = msgpack.unpackb(data_bytes)
            return self.tuple_to_obj(t)
        except Exception as e:
            print(e)
            return False

    @abstractmethod
    def tuple_to_obj(self, t):
        return False


class IData(object):
    def __init__(self, packer: IPacker) -> None:
        super().__init__()
        self.__packer = packer

    def pack(self):
        return self.__packer.pack()

    def obj_to_tuple(self):
        return self.__packer.obj_to_tuple()

    def un_pack(self, bytes_data):
        return self.__packer.un_pack(bytes_data)

    def tuple_to_obj(self, t):
        self.__packer.tuple_to_obj(t)



from ...interface import IData
from ...packer.chart.legend_item_param_data_packer import LegendItemParamDataPacker


class LegendItemParamData(IData):
    def __init__(self, plot_index: int = 0, type: int = 0, h_algin: int = 1, v_algin: int = 1, item_margin: int = 2,
                 item_space: int = 2, h_offset: int = 2, v_offset: int = 2, max_columns: int = 8, border_width: int = 1,
                 border_color: str = "", border_radius: int = 4, back_ground_alpha: int = 240, back_ground_color: str = ''):
        """图例标签配置项

        Args:
            plot_index (int, optional): 所属图块. Defaults to 0.
            type (int, optional): 类型 0 独占一块区域显示,1 与其他图叠加显示. Defaults to 0.
            h_algin (int, optional): 水平间距. Defaults to 1.
            v_algin (int, optional): 垂直间距. Defaults to 1.
            item_margin (int, optional): 项目间距. Defaults to 2.
            item_space (int, optional): 项目间空格. Defaults to 2.
            h_offset (int, optional): 水平偏移量. Defaults to 2.
            v_offset (int, optional): 垂直偏移量. Defaults to 2.
            max_columns (int, optional): 每行最大项目数. Defaults to 8.
            border_width (int, optional): 边框宽度. Defaults to 1.
            border_color (str, optional): 边框颜色. Defaults to "".
            border_radius (int, optional): 边框角弧度. Defaults to 4.
            back_ground_alpha (int, optional): 背景透明度. Defaults to 240.
            back_ground_color (str, optional): 背景色. Defaults to ''.
        """
        super().__init__(LegendItemParamDataPacker(self))
        self._PlotIndex: int = plot_index
        self._Type: int = type
        self._HAlgin: int = h_algin
        self._VAlgin: int = v_algin
        self._ItemMargin: int = item_margin
        self._ItemSpace: int = item_space
        self._HOffset: int = h_offset
        self._VOffset: int = v_offset
        self._MaxColumns: int = max_columns
        self._BorderWidth: int = border_width
        self._BorderColor: str = border_color
        self._BorderRadius: int = border_radius
        self._BackGroundAlpha: int = back_ground_alpha
        self._BackGroundColor: str = back_ground_color

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value: int):
        self._PlotIndex = value

    @property
    def Type(self):
        return self._Type

    @Type.setter
    def Type(self, value: int):
        self._Type = value

    @property
    def HAlgin(self):
        return self._HAlgin

    @HAlgin.setter
    def HAlgin(self, value: int):
        self._HAlgin = value

    @property
    def VAlgin(self):
        return self._VAlgin

    @VAlgin.setter
    def VAlgin(self, value: int):
        self._VAlgin = value

    @property
    def ItemMargin(self):
        return self._ItemMargin

    @ItemMargin.setter
    def ItemMargin(self, value: int):
        self._ItemMargin = value

    @property
    def ItemSpace(self):
        return self._ItemSpace

    @ItemSpace.setter
    def ItemSpace(self, value: int):
        self._ItemSpace = value

    @property
    def HOffset(self):
        return self._HOffset

    @HOffset.setter
    def HOffset(self, value: int):
        self._HOffset = value

    @property
    def VOffset(self):
        return self._VOffset

    @VOffset.setter
    def VOffset(self, value: int):
        self._VOffset = value

    @property
    def MaxColumns(self):
        return self._MaxColumns

    @MaxColumns.setter
    def MaxColumns(self, value: int):
        self._MaxColumns = value

    @property
    def BorderWidth(self):
        return self._BorderWidth

    @BorderWidth.setter
    def BorderWidth(self, value: int):
        self._BorderWidth = value

    @property
    def BorderColor(self):
        return self._BorderColor

    @BorderColor.setter
    def BorderColor(self, value: str):
        self._BorderColor = value

    @property
    def BorderRadius(self):
        return self._BorderRadius

    @BorderRadius.setter
    def BorderRadius(self, value: int):
        self._BorderRadius = value

    @property
    def BackGroundAlpha(self):
        return self._BackGroundAlpha

    @BackGroundAlpha.setter
    def BackGroundAlpha(self, value: int):
        self._BackGroundAlpha = value

    @property
    def BackGroundColor(self):
        return self._BackGroundColor

    @BackGroundColor.setter
    def BackGroundColor(self, value: str):
        self._BackGroundColor = value



from ...interface import IPacker


class AccountDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.AccountID), float(self._obj.PreBalance), float(self._obj.PreCredit), float(self._obj.PreMortgage),
                float(self._obj.Mortgage), float(self._obj.Withdraw), float(self._obj.Deposit), float(self._obj.StaticBalance),
                float(self._obj.CloseProfit), float(self._obj.PositionProfit), float(self._obj.Commission), float(self._obj.Balance),
                float(self._obj.CurrMargin), float(self._obj.MaintenanceMargin), float(self._obj.DeliveryMargin), float(self._obj.FrozenMargin),
                float(self._obj.FrozenCommission), float(self._obj.FrozenCash), float(self._obj.CreditLimit), float(self._obj.Available),
                float(self._obj.RoyaltyIn), float(self._obj.RoyaltyOut), float(self._obj.FrozenRoyalty), float(self._obj.OrderCommission),
                float(self._obj.RoyaltyPositionProfit), float(self._obj.MarketBalance), float(self._obj.MarketRiskDegree), float(self._obj.RiskDegree),
                str(self._obj.CurrencyNo)]

    def tuple_to_obj(self, t):
        if len(t) >= 29:
            self._obj.AccountID = t[0]
            self._obj.PreBalance = t[1]
            self._obj.PreCredit = t[2]
            self._obj.PreMortgage = t[3]
            self._obj.Mortgage = t[4]
            self._obj.Withdraw = t[5]
            self._obj.Deposit = t[6]
            self._obj.StaticBalance = t[7]
            self._obj.CloseProfit = t[8]
            self._obj.PositionProfit = t[9]
            self._obj.Commission = t[10]
            self._obj.Balance = t[11]
            self._obj.CurrMargin = t[12]
            self._obj.MaintenanceMargin = t[13]
            self._obj.DeliveryMargin = t[14]
            self._obj.FrozenMargin = t[15]
            self._obj.FrozenCommission = t[16]
            self._obj.FrozenCash = t[17]
            self._obj.CreditLimit = t[18]
            self._obj.Available = t[19]
            self._obj.RoyaltyIn = t[20]
            self._obj.RoyaltyOut = t[21]
            self._obj.FrozenRoyalty = t[22]
            self._obj.OrderCommission = t[23]
            self._obj.RoyaltyPositionProfit = t[24]
            self._obj.MarketBalance = t[25]
            self._obj.MarketRiskDegree = t[26]
            self._obj.RiskDegree = t[27]
            self._obj.CurrencyNo = t[28]

            return True
        return False



from ...interface import IPacker


class ValueAxisParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [int(self._obj.PlotIndex), int(self._obj.ValueAxisID), int(self._obj.MaxTickNum), float(self._obj.Steps),
                str(self._obj.Format), int(self._obj.LabelTextLen), float(self._obj.ValidMul), float(self._obj.PriceTick)]

    def tuple_to_obj(self, t):
        if len(t) >= 8:
            self._obj.PlotIndex = t[0]
            self._obj.ValueAxisID = t[1]
            self._obj.MaxTickNum = t[2]
            self._obj.Steps = t[3]
            self._obj.Format = t[4]
            self._obj.LabelTextLen = t[5]
            self._obj.ValidMul = t[6]
            self._obj.PriceTick = t[7]

            return True
        return False



from typing import List
from ...interface import IData
from .order_data import OrderData
from ...packer.trade.get_orders_param_data_packer import GetOrdersParamDataPacker


class GetOrdersParamData(IData):
    def __init__(self, trade_name: str = ''):
        super().__init__(GetOrdersParamDataPacker(self))
        self._TradeName = trade_name
        self._DataList: List[OrderData] = []

    @property
    def TradeName(self):
        return self._TradeName

    @TradeName.setter
    def TradeName(self, value: str):
        self._TradeName = value

    @property
    def DataList(self):
        return self._DataList

    @DataList.setter
    def DataList(self, value: List[OrderData]):
        self._DataList: List[OrderData] = value



from typing import List, Dict, Tuple
from ...interface import IData
from ...packer.chart.time_span_gvlist_data_packer import TimeSpanGVListDataPacker


class TimeSpanGVListData(IData):
    def __init__(self, time_spans: List[int], graph_values: Dict[str, List[float]] = {},
                 ohlc_values: Dict[str, Tuple[List[float], List[float], List[float], List[float]]] = {}):
        """时间戳图数据

        Args:
            time_spans (List[int]): 时间戳列表
            graph_values (Dict[str, List[float]], optional): 图数据key为图唯一编码,value为值列表. Defaults to {}.
            ohlc_values (Dict[str, Tuple[List[float], List[float], List[float], List[float]]], optional):
            ohlc数据.key为图唯一编码,value 0开盘价列表 1最高价列表 2最低价列表 3收盘价列表 Defaults to {}.
        """
        super().__init__(TimeSpanGVListDataPacker(self))
        if time_spans is None:
            time_spans = []
        self._TimeSpanList: List[int] = time_spans
        if graph_values is None:
            graph_values = {}
        self._GraphValueList: Dict[str, List[float]] = graph_values
        if ohlc_values is None:
            ohlc_values = {}
        self._OHLCValueList: Dict[str, Tuple[List[float], List[float], List[float], List[float]]] = ohlc_values

    @ property
    def TimeSpanList(self):
        return self._TimeSpanList

    @ TimeSpanList.setter
    def TimeSpanList(self, value: List[int]):
        self._TimeSpanList = value

    @ property
    def GraphValueList(self):
        return self._GraphValueList

    @ GraphValueList.setter
    def GraphValueList(self, value: Dict[str, List[float]]):
        self._GraphValueList = value

    @ property
    def OHLCValueList(self):
        return self._OHLCValueList

    @ OHLCValueList.setter
    def OHLCValueList(self, value: Dict[str, Tuple[List[float], List[float], List[float], List[float]]]):
        self._OHLCValueList = value



from ...interface import IData
from ...packer.trade.order_data_packer import OrderDataPacker


class OrderData(IData):
    def __init__(self, investor_id: str = '', broker_id: str = '', exchange_id: str = '', product_id: str = '', product_type: int = 0,
                 instrument_id: str = '', order_time: str = '', cancel_time: str = '',
                 trading_day: str = '', insert_date: str = '', update_time: str = '', status_msg: str = '',
                 front_id: int = 0, session_id: int = 0, order_ref: str = '', order_local_no: str = '',
                 order_id: str = '', relative_order_sysid: str = '', broker_order_seq: str = '', price: float = 0.0,
                 stop_price: float = 0.0, volume: int = 0, no_traded_volume: int = 0, status: int = 11,
                 direction: int = 1, open_close_type: int = 0, price_cond: int = 1, time_cond: int = 3, volume_cond: int = 0,
                 hedge_type: int = 0, order_type: int = 0, action_type: int = 0,
                 contingent_cond: int = 0, frozen_margin_price: float = 0.0, frozen_margin: float = 0.0, frozen_commission: float = 0.0,
                 show_volume: int = 0, min_volume: int = 0,
                 price_precision: int = 0, fin_biz_no: str = '', fin_algo_no: str = '', fin_insert_local_time: int = 0,
                 fin_rtn_order_local_time: int = 0, fin_lock_no: str = '',
                 biz_name: str = '', rtn_order_local_time: int = 0):
        super().__init__(OrderDataPacker(self))
        self._InvestorID: str = investor_id
        self._BrokerID: str = broker_id
        self._ExchangeID: str = exchange_id
        self._ProductID: str = product_id
        self._ProductType: int = product_type
        self._InstrumentID: str = instrument_id
        self._OrderTime: str = order_time
        self._CancelTime: str = cancel_time
        self._TradingDay: str = trading_day
        self._InsertDate: str = insert_date
        self._UpdateTime: str = update_time
        self._StatusMsg: str = status_msg
        self._FrontID: int = front_id
        self._SessionID: int = session_id
        self._OrderRef: str = order_ref
        self._OrderLocalNo: str = order_local_no
        self._OrderID: str = order_id
        self._RelativeOrderSysID: str = relative_order_sysid
        self._BrokerOrderSeq: str = broker_order_seq
        self._Price: float = price
        self._StopPrice: float = stop_price
        self._Volume: int = volume
        self._NoTradedVolume: int = no_traded_volume
        self._Status: int = status
        self._Direction: int = direction
        self._OpenCloseType: int = open_close_type
        self._PriceCond: int = price_cond
        self._TimeCond: int = time_cond
        self._VolumeCond: int = volume_cond
        self._HedgeType: int = hedge_type
        self._OrderType: int = order_type
        self._ActionType: int = action_type
        self._ContingentCond: int = contingent_cond
        self._FrozenMarginPrice: float = frozen_margin_price
        self._FrozenMargin: float = frozen_margin
        self._FrozenCommission: float = frozen_commission
        self._ShowVolume: int = show_volume
        self._MinVolume: int = min_volume
        self._PricePrecision: int = price_precision
        self._FinBizNo: str = fin_biz_no
        self._FinAlgoNo: str = fin_algo_no
        self._FinInsertLocalTime: int = fin_insert_local_time
        self._FinRtnOrderLocalTime: int = fin_rtn_order_local_time
        self._FinLockNo: str = fin_lock_no
        self._BizName: str = biz_name
        self._RtnOrderLocalTime: int = rtn_order_local_time

    @property
    def InvestorID(self):
        return self._InvestorID

    @InvestorID.setter
    def InvestorID(self, value: str):
        self._InvestorID = value

    @property
    def BrokerID(self):
        return self._BrokerID

    @BrokerID.setter
    def BrokerID(self, value: str):
        self._BrokerID = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def ProductID(self):
        return self._ProductID

    @ProductID.setter
    def ProductID(self, value: str):
        self._ProductID = value

    @property
    def ProductType(self):
        return self._ProductType

    @ProductType.setter
    def ProductType(self, value: int):
        self._ProductType = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def OrderTime(self):
        return self._OrderTime

    @OrderTime.setter
    def OrderTime(self, value: str):
        self._OrderTime = value

    @property
    def CancelTime(self):
        return self._CancelTime

    @CancelTime.setter
    def CancelTime(self, value: str):
        self._CancelTime = value

    @property
    def TradingDay(self):
        return self._TradingDay

    @TradingDay.setter
    def TradingDay(self, value: str):
        self._TradingDay = value

    @property
    def InsertDate(self):
        return self._InsertDate

    @InsertDate.setter
    def InsertDate(self, value: str):
        self._InsertDate = value

    @property
    def UpdateTime(self):
        return self._UpdateTime

    @UpdateTime.setter
    def UpdateTime(self, value: str):
        self._UpdateTime = value

    @property
    def StatusMsg(self):
        return self._StatusMsg

    @StatusMsg.setter
    def StatusMsg(self, value: str):
        self._StatusMsg = value

    @property
    def FrontID(self):
        return self._FrontID

    @FrontID.setter
    def FrontID(self, value: int):
        self._FrontID = value

    @property
    def SessionID(self):
        return self._SessionID

    @SessionID.setter
    def SessionID(self, value: int):
        self._SessionID = value

    @property
    def OrderRef(self):
        return self._OrderRef

    @OrderRef.setter
    def OrderRef(self, value: str):
        self._OrderRef = value

    @property
    def OrderLocalNo(self):
        return self._OrderLocalNo

    @OrderLocalNo.setter
    def OrderLocalNo(self, value: str):
        self._OrderLocalNo = value

    @property
    def OrderID(self):
        return self._OrderID

    @OrderID.setter
    def OrderID(self, value: str):
        self._OrderID = value

    @property
    def RelativeOrderSysID(self):
        return self._RelativeOrderSysID

    @RelativeOrderSysID.setter
    def RelativeOrderSysID(self, value: str):
        self._RelativeOrderSysID = value

    @property
    def BrokerOrderSeq(self):
        return self._BrokerOrderSeq

    @BrokerOrderSeq.setter
    def BrokerOrderSeq(self, value: str):
        self._BrokerOrderSeq = value

    @property
    def Price(self):
        return self._Price

    @Price.setter
    def Price(self, value: float):
        self._Price = value

    @property
    def StopPrice(self):
        return self._StopPrice

    @StopPrice.setter
    def StopPrice(self, value: float):
        self._StopPrice = value

    @property
    def Volume(self):
        return self._Volume

    @Volume.setter
    def Volume(self, value: int):
        self._Volume = value

    @property
    def NoTradedVolume(self):
        return self._NoTradedVolume

    @NoTradedVolume.setter
    def NoTradedVolume(self, value: int):
        self._NoTradedVolume = value

    @property
    def Status(self):
        return self._Status

    @Status.setter
    def Status(self, value: int):
        self._Status = value

    @property
    def Direction(self):
        return self._Direction

    @Direction.setter
    def Direction(self, value: int):
        self._Direction = value

    @property
    def OpenCloseType(self):
        return self._OpenCloseType

    @OpenCloseType.setter
    def OpenCloseType(self, value: int):
        self._OpenCloseType = value

    @property
    def PriceCond(self):
        return self._PriceCond

    @PriceCond.setter
    def PriceCond(self, value: int):
        self._PriceCond = value

    @property
    def TimeCond(self):
        return self._TimeCond

    @TimeCond.setter
    def TimeCond(self, value: int):
        self._TimeCond = value

    @property
    def VolumeCond(self):
        return self._VolumeCond

    @VolumeCond.setter
    def VolumeCond(self, value: int):
        self._VolumeCond = value

    @property
    def HedgeType(self):
        return self._HedgeType

    @HedgeType.setter
    def HedgeType(self, value: int):
        self._HedgeType = value

    @property
    def OrderType(self):
        return self._OrderType

    @OrderType.setter
    def OrderType(self, value: int):
        self._OrderType = value

    @property
    def ActionType(self):
        return self._ActionType

    @ActionType.setter
    def ActionType(self, value: int):
        self._ActionType = value

    @property
    def ContingentCond(self):
        return self._ContingentCond

    @ContingentCond.setter
    def ContingentCond(self, value: int):
        self._ContingentCond = value

    @property
    def FrozenMarginPrice(self):
        return self._FrozenMarginPrice

    @FrozenMarginPrice.setter
    def FrozenMarginPrice(self, value: float):
        self._FrozenMarginPrice = value

    @property
    def FrozenMargin(self):
        return self._FrozenMargin

    @FrozenMargin.setter
    def FrozenMargin(self, value: float):
        self._FrozenMargin = value

    @property
    def FrozenCommission(self):
        return self._FrozenCommission

    @FrozenCommission.setter
    def FrozenCommission(self, value: float):
        self._FrozenCommission = value

    @property
    def ShowVolume(self):
        return self._ShowVolume

    @ShowVolume.setter
    def ShowVolume(self, value: int):
        self._ShowVolume = value

    @property
    def MinVolume(self):
        return self._MinVolume

    @MinVolume.setter
    def MinVolume(self, value: int):
        self._MinVolume = value

    @property
    def PricePrecision(self):
        return self._PricePrecision

    @PricePrecision.setter
    def PricePrecision(self, value: int):
        self._PricePrecision = value

    @property
    def FinBizNo(self):
        return self._FinBizNo

    @FinBizNo.setter
    def FinBizNo(self, value: str):
        self._FinBizNo = value

    @property
    def FinAlgoNo(self):
        return self._FinAlgoNo

    @FinAlgoNo.setter
    def FinAlgoNo(self, value: str):
        self._FinAlgoNo = value

    @property
    def FinInsertLocalTime(self):
        return self._FinInsertLocalTime

    @FinInsertLocalTime.setter
    def FinInsertLocalTime(self, value: int):
        self._FinInsertLocalTime = value

    @property
    def FinRtnOrderLocalTime(self):
        return self._FinRtnOrderLocalTime

    @FinRtnOrderLocalTime.setter
    def FinRtnOrderLocalTime(self, value: int):
        self._FinRtnOrderLocalTime = value

    @property
    def FinLockNo(self):
        return self._FinLockNo

    @FinLockNo.setter
    def FinLockNo(self, value: str):
        self._FinLockNo = value

    @property
    def BizName(self):
        return self._BizName

    @BizName.setter
    def BizName(self, value: str):
        self._BizName = value

    @property
    def RtnOrderLocalTime(self):
        return self._RtnOrderLocalTime

    @RtnOrderLocalTime.setter
    def RtnOrderLocalTime(self, value: int):
        self._RtnOrderLocalTime = value



from typing import List, Dict, Tuple
import datetime
from .req_rsp import ReqRspDict, ReqRsp
from ..interface import IData, MsgID
from ..tsocket import TSocket
from ..data.message_data import MessageData
from ..data.chart.chart_init_param_data import ChartInitParamData
from ..data.chart.line_graph_param_data import LineGraphParamData
from ..data.chart.financial_graph_param_data import FinancialGraphParamData
from ..data.chart.text_graph_param_data import TextGraphParamData
from ..data.chart.marker_graph_param_data import MarkerGraphParamData
from ..data.chart.ohlc_value_list_data import OHLCValueListData
from ..data.chart.ohlc_value_data import OHLCValueData
from ..data.chart.graph_value_list_data import GraphValueListData
from ..data.chart.graph_value_data import GraphValueData
from ..data.chart.bar_graph_param_data import BarGraphParamData
from ..data.chart.time_span_gvlist_data import TimeSpanGVListData


class ChartHandle():
    __ReqID: int = 0
    __ReqRspDict: ReqRspDict = ReqRspDict()

    def __init__(self, tsocket: TSocket):
        self.__TSocket = tsocket
        self.__TSocket.set_chart_callback(self.__recv_msg)

    def set_chart_init_params(self, params: ChartInitParamData):
        return self.__wait_send_msg(int(MsgID.Chart_Set_Init_Param.value), params)

    def add_line_graph(self, params: LineGraphParamData):
        return self.__wait_send_msg(int(MsgID.Chart_AddLineGraph.value), params)

    def add_bar_graph(self, params: BarGraphParamData):
        return self.__wait_send_msg(int(MsgID.Chart_AddBarGraph.value), params)

    def add_financial_graph(self, params: FinancialGraphParamData):
        return self.__wait_send_msg(int(MsgID.Chart_AddFinancialGraph.value), params)

    def add_marker_graph(self, param: MarkerGraphParamData):
        return self.__wait_send_msg(int(MsgID.Chart_AddMarkerGraph.value), param)

    def add_text_graph(self, params: TextGraphParamData):
        return self.__wait_send_msg(int(MsgID.Chart_AddTextGraph.value), params)

    def chart_init_show(self):
        return self.__wait_send_msg(int(MsgID.Chart_Init_Show.value), None)

    def add_graph_value(self, gvd: GraphValueData):
        return self.__only_send_msg(int(MsgID.Chart_AddGraphValue.value), gvd)

    def add_ohlc_value(self, ovd: OHLCValueData):
        return self.__only_send_msg(int(MsgID.Chart_AddOHLCValue.value), ovd)

    def save_chart_data(self, filename: str):
        return self.__wait_send_msg(int(MsgID.Chart_SaveChartData.value), filename)

    def load_chart_data(self, filename: str):
        return self.__wait_send_msg(int(MsgID.Chart_LoadChartData.value), filename)

    def add_graph_value_list(self, gv_value_list: List[GraphValueData]):
        data = GraphValueListData(gv_list=gv_value_list)
        return self.__only_send_msg(int(MsgID.Chart_AddGraphValueList.value), data)

    def add_timespan_graphvalue_list(self, timespans: List[int], graphvalues: Dict[str, List[float]] = {},
                                     ohlcvalues: Dict[str, Tuple[List[float], List[float], List[float], List[float]]] = {}):
        data = TimeSpanGVListData(time_spans=timespans, graph_values=graphvalues, ohlc_values=ohlcvalues)
        return self.__only_send_msg(int(MsgID.Chart_AddTimeSpanGVList.value), data)

    def add_ohlc_value_list(self, ohlc_value_list: List[OHLCValueData]):
        data = OHLCValueListData(ohlc_value_list=ohlc_value_list)
        return self.__only_send_msg(int(MsgID.Chart_AddOHLCValueList.value), data)

    def __only_send_msg(self, mid, params: IData,) -> Tuple[bool, str]:
        self.__ReqID = self.__ReqID + 1
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            msg.UData = params.pack()

        if self.__TSocket.send_message(msg) is False:
            return [False, '发送失败']

        return [True, '发送成功']

    def __recv_msg(self, msg: MessageData):
        key = '%s_%s' % (msg.MID, msg.RequestID)
        reqrsp: ReqRsp = self.__ReqRspDict.get_reqrsp(key)
        if reqrsp is not None:
            reqrsp.append_rsp(msg)

    def __wait_send_msg(self, mid, params):
        self.__ReqID = self.__ReqID + 1
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            if isinstance(params, IData):
                msg.UData = params.pack()
            elif isinstance(params, str):
                msg.UData = bytes(params, 'utf-8')

        key = '%s_%s' % (mid, self.__ReqID)

        req_rsp = self.__ReqRspDict.new_reqrsp(key, msg)
        if self.__TSocket.send_message(msg) is False:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令失败']

        rsp = req_rsp.wait_last_rsp(10)
        if rsp is None:
            self.__ReqRspDict.remove(key)
            return [False, '发送超时']

        ret = [rsp.RspSuccess, rsp.RspMsg]
        self.__ReqRspDict.remove(key)
        return ret



from ..interface import IPacker


class MessageDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [int(self._obj.MID), int(self._obj.ActionID), int(self._obj.RequestID), bool(self._obj.IsLast),
                bool(self._obj.RspSuccess), str(self._obj.RspMsg), bytes(self._obj.UData), int(self._obj.CompressType)]

    def tuple_to_obj(self, t):
        if len(t) >= 8:
            self._obj.MID = t[0]
            self._obj.ActionID = t[1]
            self._obj.RequestID = t[2]
            self._obj.IsLast = t[3]
            self._obj.RspSuccess = t[4]
            self._obj.RspMsg = t[5]
            self._obj.UData = t[6]
            self._obj.CompressType = t[7]
            return True
        return False



from ...interface import IPacker


class GraphValueDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.GraphID), float(self._obj.Key), int(self._obj.MillTimeSpan), float(self._obj.Value)]

    def tuple_to_obj(self, t):
        if len(t) >= 4:
            self._obj.GraphID = t[0]
            self._obj.Key = t[1]
            self._obj.MillTimeSpan = t[2]
            self._obj.Value = t[3]

            return True
        return False



from ...interface import IPacker


class OHLCDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.ExchangeID), str(self._obj.InstrumentID), str(self._obj.TradingDay), str(self._obj.TradingTime),
                str(self._obj.StartTime), str(self._obj.EndTime), str(self._obj.ActionDay), int(self._obj.ActionTimeSpan),
                int(self._obj.Range), int(self._obj.Index), float(self._obj.OpenPrice), float(self._obj.HighestPrice),
                float(self._obj.LowestPrice), float(self._obj.ClosePrice), float(self._obj.TotalTurnover), int(self._obj.TotalVolume),
                float(self._obj.OpenInterest), float(self._obj.PreSettlementPrice), float(self._obj.ChangeRate), float(self._obj.ChangeValue),
                float(self._obj.OpenBidPrice), float(self._obj.OpenAskPrice), int(self._obj.OpenBidVolume), int(self._obj.OpenAskVolume),
                float(self._obj.HighestBidPrice), float(self._obj.HighestAskPrice), int(self._obj.HighestBidVolume), int(self._obj.HighestAskVolume),
                float(self._obj.LowestBidPrice), float(self._obj.LowestAskPrice), int(self._obj.LowestBidVolume), int(self._obj.LowestAskVolume),
                float(self._obj.CloseBidPrice), float(self._obj.CloseAskPrice), int(self._obj.CloseBidVolume), int(self._obj.CloseAskVolume)]

    def tuple_to_obj(self, t):
        if len(t) >= 36:
            self._obj.ExchangeID = t[0]
            self._obj.InstrumentID = t[1]
            self._obj.TradingDay = t[2]
            self._obj.TradingTime = t[3]
            self._obj.StartTime = t[4]
            self._obj.EndTime = t[5]
            self._obj.ActionDay = t[6]
            self._obj.ActionTimeSpan = t[7]
            self._obj.Range = t[8]
            self._obj.Index = t[9]
            self._obj.OpenPrice = t[10]
            self._obj.HighestPrice = t[11]
            self._obj.LowestPrice = t[12]
            self._obj.ClosePrice = t[13]
            self._obj.TotalTurnover = t[14]
            self._obj.TotalVolume = t[15]
            self._obj.OpenInterest = t[16]
            self._obj.PreSettlementPrice = t[17]
            self._obj.ChangeRate = t[18]
            self._obj.ChangeValue = t[19]
            self._obj.OpenBidPrice = t[20]
            self._obj.OpenAskPrice = t[21]
            self._obj.OpenBidVolume = t[22]
            self._obj.OpenAskVolume = t[23]
            self._obj.HighestBidPrice = t[24]
            self._obj.HighestAskPrice = t[25]
            self._obj.HighestBidVolume = t[26]
            self._obj.HighestAskVolume = t[27]
            self._obj.LowestBidPrice = t[28]
            self._obj.LowestAskPrice = t[29]
            self._obj.LowestBidVolume = t[30]
            self._obj.LowestAskVolume = t[31]
            self._obj.CloseBidPrice = t[32]
            self._obj.CloseAskPrice = t[33]
            self._obj.CloseBidVolume = t[34]
            self._obj.CloseAskVolume = t[35]

            return True
        return False



from typing import List
from ...interface import IData
from .graph_value_data import GraphValueData
from ...packer.chart.graph_value_list_data_packer import GraphValueListDataPacker


class GraphValueListData(IData):
    def __init__(self, gv_list: List[GraphValueData]):
        super().__init__(GraphValueListDataPacker(self))
        self._GVList: List[GraphValueData] = gv_list

    @property
    def GVList(self):
        return self._GVList

    @GVList.setter
    def GVList(self, value: List[GraphValueData]):
        self._GVList = value



from ...interface import IPacker


class QueryParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.MarketName), str(self._obj.InvestorID), str(self._obj.BrokerID), str(self._obj.ExchangeID),
                str(self._obj.ExchangeInstID), str(self._obj.InstrumentID), int(self._obj.ProductType), str(self._obj.ProductID),
                str(self._obj.OrderID), str(self._obj.TradeID), str(self._obj.InsertTimeStart), str(self._obj.InsertTimeEnd),
                str(self._obj.TradeTimeStart), str(self._obj.TradeTimeEnd), str(self._obj.CurrencyID), int(self._obj.HedgeType)]

    def tuple_to_obj(self, t):
        if len(t) >= 16:
            self._obj.MarketName = t[0]
            self._obj.InvestorID = t[1]
            self._obj.BrokerID = t[2]
            self._obj.ExchangeID = t[3]
            self._obj.ExchangeInstID = t[4]
            self._obj.InstrumentID = t[5]
            self._obj.ProductType = t[6]
            self._obj.ProductID = t[7]
            self._obj.OrderID = t[8]
            self._obj.TradeID = t[9]
            self._obj.InsertTimeStart = t[10]
            self._obj.InsertTimeEnd = t[11]
            self._obj.TradeTimeStart = t[12]
            self._obj.TradeTimeEnd = t[13]
            self._obj.CurrencyID = t[14]
            self._obj.HedgeType = t[15]

            return True
        return False



from ...interface import IPacker


class TickDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.InstrumentID), str(self._obj.ExchangeID), str(self._obj.TradingDay), str(self._obj.ActionDay),
                str(self._obj.TradingTime), float(self._obj.LastPrice), int(self._obj.LastVolume), float(self._obj.AskPrice),
                int(self._obj.AskVolume), float(self._obj.BidPrice), int(self._obj.BidVolume)]

    def tuple_to_obj(self, t):
        if len(t) >= 11:
            self._obj.InstrumentID = t[0]
            self._obj.ExchangeID = t[1]
            self._obj.TradingDay = t[2]
            self._obj.ActionDay = t[3]
            self._obj.TradingTime = t[4]
            self._obj.LastPrice = t[5]
            self._obj.LastVolume = t[6]
            self._obj.AskPrice = t[7]
            self._obj.AskVolume = t[8]
            self._obj.BidPrice = t[9]
            self._obj.BidVolume = t[10]

            return True
        return False



from ...interface import IData
from ...packer.market.query_param_data_packer import QueryParamDataPacker


class QueryParamData(IData):
    def __init__(self, market_name: str = '', investor_id: str = '', broker_id: str = '', exchange_id: str = '',
                 exchange_instid: str = '', instrument_id: str = '', product_type: int = 0,
                 product_id: str = '', order_id: str = '', trade_id: str = '', insert_time_start: str = '',
                 insert_time_end: str = '', trade_time_start: str = '', trade_time_end: str = '',
                 currency_id: str = '', hedge_type: int = 0):
        super().__init__(QueryParamDataPacker(self))
        self._MarketName: str = market_name
        self._InvestorID: str = investor_id
        self._BrokerID: str = broker_id
        self._ExchangeID: str = exchange_id
        self._ExchangeInstID: str = exchange_instid
        self._InstrumentID: str = instrument_id
        self._ProductType: int = product_type
        self._ProductID: str = product_id
        self._OrderID: str = order_id
        self._TradeID: str = trade_id
        self._InsertTimeStart: str = insert_time_start
        self._InsertTimeEnd: str = insert_time_end
        self._TradeTimeStart: str = trade_time_start
        self._TradeTimeEnd: str = trade_time_end
        self._CurrencyID: str = currency_id
        self._HedgeType: int = hedge_type

    @property
    def MarketName(self):
        return self._MarketName

    @MarketName.setter
    def MarketName(self, value: str):
        self._MarketName = value

    @property
    def InvestorID(self):
        return self._InvestorID

    @InvestorID.setter
    def InvestorID(self, value: str):
        self._InvestorID = value

    @property
    def BrokerID(self):
        return self._BrokerID

    @BrokerID.setter
    def BrokerID(self, value: str):
        self._BrokerID = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def ExchangeInstID(self):
        return self._ExchangeInstID

    @ExchangeInstID.setter
    def ExchangeInstID(self, value: str):
        self._ExchangeInstID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def ProductType(self):
        return self._ProductType

    @ProductType.setter
    def ProductType(self, value: int):
        self._ProductType = value

    @property
    def ProductID(self):
        return self._ProductID

    @ProductID.setter
    def ProductID(self, value: str):
        self._ProductID = value

    @property
    def OrderID(self):
        return self._OrderID

    @OrderID.setter
    def OrderID(self, value: str):
        self._OrderID = value

    @property
    def TradeID(self):
        return self._TradeID

    @TradeID.setter
    def TradeID(self, value: str):
        self._TradeID = value

    @property
    def InsertTimeStart(self):
        return self._InsertTimeStart

    @InsertTimeStart.setter
    def InsertTimeStart(self, value: str):
        self._InsertTimeStart = value

    @property
    def InsertTimeEnd(self):
        return self._InsertTimeEnd

    @InsertTimeEnd.setter
    def InsertTimeEnd(self, value: str):
        self._InsertTimeEnd = value

    @property
    def TradeTimeStart(self):
        return self._TradeTimeStart

    @TradeTimeStart.setter
    def TradeTimeStart(self, value: str):
        self._TradeTimeStart = value

    @property
    def TradeTimeEnd(self):
        return self._TradeTimeEnd

    @TradeTimeEnd.setter
    def TradeTimeEnd(self, value: str):
        self._TradeTimeEnd = value

    @property
    def CurrencyID(self):
        return self._CurrencyID

    @CurrencyID.setter
    def CurrencyID(self, value: str):
        self._CurrencyID = value

    @property
    def HedgeType(self):
        return self._HedgeType

    @HedgeType.setter
    def HedgeType(self, value: int):
        self._HedgeType = value



from typing import Dict
from ...interface import IData
from ...packer.chart.text_graph_param_data_packer import TextGraphParamDataPacker


class TextGraphParamData(IData):
    def __init__(self, id: str = '', name: str = '', color: str = 'white', plot_index: int = 0, value_axis_id: int = -1,
                 text: str = '', key: float = 0.0, value: float = 0.0, mill_ts: int = -1, join_value_axis=True,
                 show_legend: bool = False, legend_format: str = '',
                 legend_color: str = '', user_data: Dict[str, str] = {}, visible: bool = True):
        """文字配置参数

        Args:
            id (str, optional): 图唯一编码. Defaults to ''.
            name (str, optional): 图名称. Defaults to ''.
            color (str, optional): 文字颜色. Defaults to 'white'.
            plot_index (int, optional): 图块索引. Defaults to 0.
            value_axis_id (int, optional): 所属值轴. Defaults to -1.
            text (str, optional): 文字内容. Defaults to ''.
            key (float, optional): 数值格式的x值. Defaults to 0.0.
            value (float, optional): 数值格式的y值. Defaults to 0.0.
            mill_ts (int, optional): 时间戳格式的x值. Defaults to -1.
            join_value_axis (bool, optional): 是否加入到y轴可视范围的计算中. Defaults to True.
            show_legend (bool, optional): 是否显示标签. Defaults to True.
            legend_format (str, optional): 标签格式. Defaults to ''.
            legend_color (str, optional): 标签颜色,默认与图颜色相同. Defaults to ''.
            user_data (Dict[str, str], optional): 用户自定义数据. Defaults to {}.
            visible (bool, optional): 是否显示 Defaults to True.
        """
        super().__init__(TextGraphParamDataPacker(self))
        self._ID = id
        self._Name = name
        self._Color = color
        self._PlotIndex = plot_index
        self._ValueAxisID = value_axis_id
        self._Text = text
        self._Key = key
        self._Value = value
        self._MillTimeSpan = mill_ts
        self._JoinValueAxis: bool = join_value_axis
        self._ShowLegend: bool = show_legend
        self._LegendFormat: str = legend_format
        self._LegendColor: str = legend_color
        self._UserData: Dict[str, str] = user_data
        self._Visible: bool = visible

    @property
    def ID(self):
        return self._ID

    @ID.setter
    def ID(self, value: str):
        self._ID = value

    @property
    def Name(self):
        return self._Name

    @Name.setter
    def Name(self, value: str):
        self._Name = value

    @property
    def Color(self):
        return self._Color

    @Color.setter
    def Color(self, value: str):
        self._Color = value

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value: int):
        self._PlotIndex = value

    @property
    def ValueAxisID(self):
        return self._ValueAxisID

    @ValueAxisID.setter
    def ValueAxisID(self, value: int):
        self._ValueAxisID = value

    @property
    def Text(self):
        return self._Text

    @Text.setter
    def Text(self, value: str):
        self._Text = value

    @property
    def Key(self):
        return self._Key

    @Key.setter
    def Key(self, value: float):
        self._Key = value

    @property
    def Value(self):
        return self._Value

    @Value.setter
    def Value(self, value: float):
        self._Value = value

    @property
    def MillTimeSpan(self):
        return self._MillTimeSpan

    @MillTimeSpan.setter
    def MillTimeSpan(self, value: int):
        self._MillTimeSpan = value

    @property
    def JoinValueAxis(self):
        return self._JoinValueAxis

    @JoinValueAxis.setter
    def JoinValueAxis(self, value: bool):
        self._JoinValueAxis = value

    @property
    def ShowLegend(self):
        return self._ShowLegend

    @ShowLegend.setter
    def ShowLegend(self, value):
        self._ShowLegend = (value)

    @property
    def LegendFormat(self):
        return self._LegendFormat

    @LegendFormat.setter
    def LegendFormat(self, value):
        self._LegendFormat = str(value)

    @property
    def LegendColor(self):
        return self._LegendColor

    @LegendColor.setter
    def LegendColor(self, value):
        self._LegendColor = str(value)

    @property
    def UserData(self):
        return self._UserData

    @UserData.setter
    def UserData(self, value: Dict[str, str]):
        self._UserData = value

    @property
    def Visible(self):
        return self._Visible

    @JoinValueAxis.setter
    def Visible(self, value: bool):
        self._Visible = value



from typing import List, Dict, Tuple
from ...interface import IData
from ...packer.chart.chart_init_param_data_packer import ChartInitParamDataPacker
from .time_axis_param_data import TimeAxisParamData
from .value_axis_param_data import ValueAxisParamData
from .plot_param_data import PlotParamData
from .legend_item_param_data import LegendItemParamData
from .line_graph_param_data import LineGraphParamData
from .financial_graph_param_data import FinancialGraphParamData
from .bar_graph_param_data import BarGraphParamData
from .text_graph_param_data import TextGraphParamData
from .marker_graph_param_data import MarkerGraphParamData


class ChartInitParamData(IData):
    def __init__(self):
        super().__init__(ChartInitParamDataPacker(self))
        self._ChartID = ''
        self._Title = ''
        self._Height = 600
        self._Width = 800
        self._IsSaveGeometry = True
        self._ReplotTime = 100
        self._GlobalTimeAxisParam = TimeAxisParamData()
        self._GlobalValueAxisParam = ValueAxisParamData()
        self._GlobalPlotParam = PlotParamData()
        self._GlobalLegendItemParam = LegendItemParamData()
        self._PlotParamList: List[PlotParamData] = []
        self._ValueAxisParamList: List[ValueAxisParamData] = []
        self._LegendItemParamList: List[LegendItemParamData] = []
        self._FinancialGraphParamList: List[FinancialGraphParamData] = []
        self._LineGraphParamList: List[LineGraphParamData] = []
        self._BarGraphParamList: List[BarGraphParamData] = []
        self._TextGraphParamList: List[TextGraphParamData] = []
        self._MarkerGraphParamList: List[MarkerGraphParamData] = []
        self._TimeSpanList: List[int] = []
        self._GraphValueList: Dict[str, List[float]] = {}
        self._OHLCValueList: Dict[str, Tuple[List[float], List[float], List[float], List[float]]] = {}

    @property
    def ChartID(self):
        return self._ChartID

    @ChartID.setter
    def ChartID(self, value: str):
        self._ChartID = value

    @property
    def Title(self):
        return self._Title

    @Title.setter
    def Title(self, value: str):
        self._Title = value

    @property
    def Height(self):
        return self._Height

    @Height.setter
    def Height(self, value: int):
        self._Height = value

    @property
    def Width(self):
        return self._Width

    @Width.setter
    def Width(self, value: int):
        self._Width = value

    @property
    def IsSaveGeometry(self):
        return self._IsSaveGeometry

    @IsSaveGeometry.setter
    def IsSaveGeometry(self, value: bool):
        self._IsSaveGeometry = value

    @property
    def ReplotTime(self):
        return self._ReplotTime

    @ReplotTime.setter
    def ReplotTime(self, value: int):
        self._ReplotTime = value

    @property
    def GlobalTimeAxisParam(self):
        return self._GlobalTimeAxisParam

    @GlobalTimeAxisParam.setter
    def GlobalTimeAxisParam(self, value: TimeAxisParamData):
        self._GlobalTimeAxisParam = value

    @property
    def GlobalValueAxisParam(self):
        return self._GlobalValueAxisParam

    @GlobalValueAxisParam.setter
    def GlobalValueAxisParam(self, value: ValueAxisParamData):
        self._GlobalValueAxisParam = value

    @property
    def GlobalPlotParam(self):
        return self._GlobalPlotParam

    @GlobalPlotParam.setter
    def GlobalPlotParam(self, value: PlotParamData):
        self._GlobalPlotParam = value

    @property
    def GlobalLegendItemParam(self):
        return self._GlobalLegendItemParam

    @GlobalLegendItemParam.setter
    def GlobalLegendItemParam(self, value: LegendItemParamData):
        self._GlobalLegendItemParam = value

    @property
    def PlotParamList(self):
        return self._PlotParamList

    @PlotParamList.setter
    def PlotParamList(self, value: List[PlotParamData]):
        self._PlotParamList = value

    @property
    def ValueAxisParamList(self):
        return self._ValueAxisParamList

    @ValueAxisParamList.setter
    def ValueAxisParamList(self, value: List[ValueAxisParamData]):
        self._ValueAxisParamList = value

    @property
    def LegendItemParamList(self):
        return self._LegendItemParamList

    @LegendItemParamList.setter
    def LegendItemParamList(self, value: List[LegendItemParamData]):
        self._LegendItemParamList = value

    @property
    def FinancialGraphParamList(self):
        return self._FinancialGraphParamList

    @FinancialGraphParamList.setter
    def FinancialGraphParamList(self, value: List[FinancialGraphParamData]):
        self._FinancialGraphParamList = value

    @property
    def LineGraphParamList(self):
        return self._LineGraphParamList

    @LineGraphParamList.setter
    def LineGraphParamList(self, value: List[LineGraphParamData]):
        self._LineGraphParamList = value

    @property
    def BarGraphParamList(self):
        return self._BarGraphParamList

    @BarGraphParamList.setter
    def BarGraphParamList(self, value: List[BarGraphParamData]):
        self._BarGraphParamList = value

    @property
    def TextGraphParamList(self):
        return self._TextGraphParamList

    @TextGraphParamList.setter
    def TextGraphParamList(self, value: List[TextGraphParamData]):
        self._TextGraphParamList = value

    @property
    def MarkerGraphParamList(self):
        return self._MarkerGraphParamList

    @MarkerGraphParamList.setter
    def MarkerGraphParamList(self, value: List[MarkerGraphParamData]):
        self._MarkerGraphParamList = value

    @ property
    def TimeSpanList(self):
        return self._TimeSpanList

    @ TimeSpanList.setter
    def TimeSpanList(self, value: List[int]):
        self._TimeSpanList = value

    @ property
    def GraphValueList(self):
        return self._GraphValueList

    @ GraphValueList.setter
    def GraphValueList(self, value: Dict[str, List[float]]):
        self._GraphValueList = value

    @ property
    def OHLCValueList(self):
        return self._OHLCValueList

    @ OHLCValueList.setter
    def OHLCValueList(self, value: Dict[str, Tuple[List[float], List[float], List[float], List[float]]]):
        self._OHLCValueList = value



from ...interface import IData
from ...packer.chart.value_axis_param_data_packer import ValueAxisParamDataPacker


class ValueAxisParamData(IData):
    def __init__(self, plot_index: int = 0, value_axis_id: int = -1, max_tick_num: int = 6, steps: float = -1.0,
                 format: str = '', label_text_len: int = 6, valid_mul: float = -1.0, price_tick: float = 1.0):
        """value (y) 轴配置参数

        Args:
            plot_index (int, optional): 所属区块. Defaults to 0.
            value_axis_id (int, optional): 左侧第一个Y轴. Defaults to -1.
            max_tick_num (int, optional): 最大刻度标签数. Defaults to 6.
            steps (float, optional): 刻度固定间隔. Defaults to -1.0.
            format (str, optional): 标签格式化. Defaults to ''.
            label_text_len (int, optional): 标签最大数字宽度. Defaults to 6.
            valid_mul (float, optional): 有效倍数. Defaults to -1.0.
            price_tick (float, optional): 最小刻度变动值. Defaults to 1.0.
        """
        super().__init__(ValueAxisParamDataPacker(self))

        self._PlotIndex: int = plot_index
        self._ValueAxisID: int = value_axis_id
        self._MaxTickNum: int = max_tick_num
        self._Steps: float = steps
        self._Format: str = format
        self._LabelTextLen: int = label_text_len
        self._ValidMul: float = valid_mul
        self._PriceTick: float = price_tick

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value: int):
        self._PlotIndex = value

    @property
    def ValueAxisID(self):
        return self._ValueAxisID

    @ValueAxisID.setter
    def ValueAxisID(self, value: int):
        self._ValueAxisID = value

    @property
    def MaxTickNum(self):
        return self._MaxTickNum

    @MaxTickNum.setter
    def MaxTickNum(self, value: int):
        self._MaxTickNum = value

    @property
    def Steps(self):
        return self._Steps

    @Steps.setter
    def Steps(self, value: float):
        self._Steps = value

    @property
    def Format(self):
        return self._Format

    @Format.setter
    def Format(self, value: str):
        self._Format = value

    @property
    def LabelTextLen(self):
        return self._LabelTextLen

    @LabelTextLen.setter
    def LabelTextLen(self, value: int):
        self._LabelTextLen = value

    @property
    def ValidMul(self):
        return self._ValidMul

    @ValidMul.setter
    def ValidMul(self, value: float):
        self._ValidMul = value

    @property
    def PriceTick(self):
        return self._PriceTick

    @PriceTick.setter
    def PriceTick(self, value: float):
        self._PriceTick = value



from ...interface import IPacker


class OHLCValueListDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        ohlc_list = []
        for o in self._obj.OHLCList:
            ohlc_list.append(o.obj_to_tuple())
        return [ohlc_list]

    def tuple_to_obj(self, t):
        if len(t) >= 1:
            self._obj.OHLCList = t[0]

            return True
        return False



from ...interface import IPacker


class LegendItemParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [int(self._obj.PlotIndex), int(self._obj.Type), int(self._obj.HAlgin),
                int(self._obj.VAlgin), int(self._obj.ItemMargin),
                int(self._obj.ItemSpace), int(self._obj.HOffset),
                int(self._obj.VOffset), int(self._obj.MaxColumns),
                int(self._obj.BorderWidth), str(self._obj.BorderColor),
                int(self._obj.BorderRadius), int(self._obj.BackGroundAlpha),
                str(self._obj.BackGroundColor)]

    def tuple_to_obj(self, t):
        if len(t) >= 14:
            self._obj.PlotIndex = t[0]
            self._obj.Type = t[1]
            self._obj.HAlgin = t[2]
            self._obj.VAlgin = t[3]
            self._obj.ItemMargin = t[4]
            self._obj.ItemSpace = t[5]
            self._obj.HOffset = t[6]
            self._obj.VOffset = t[7]
            self._obj.MaxColumns = t[8]
            self._obj.BorderWidth = t[9]
            self._obj.BorderColor = t[10]
            self._obj.BorderRadius = t[11]
            self._obj.BackGroundAlpha = t[12]
            self._obj.BackGroundColor = t[13]

            return True
        return False



from ...interface import IPacker


class ChartInitParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    @staticmethod
    def objlist_to_tuplelist(objlist):
        retlist = list()
        for obj in objlist:
            retlist.append(obj.obj_to_tuple())
        return retlist

    def obj_to_tuple(self):
        return [str(self._obj.ChartID), str(self._obj.Title), int(self._obj.Height), int(self._obj.Width),
                bool(self._obj.IsSaveGeometry), int(self._obj.ReplotTime),
                self._obj.GlobalTimeAxisParam.obj_to_tuple(),
                self._obj.GlobalValueAxisParam.obj_to_tuple(),
                self._obj.GlobalPlotParam.obj_to_tuple(),
                self._obj.GlobalLegendItemParam.obj_to_tuple(),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.PlotParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.ValueAxisParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.LegendItemParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.LineGraphParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.FinancialGraphParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.BarGraphParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.TextGraphParamList),
                ChartInitParamDataPacker.objlist_to_tuplelist(self._obj.MarkerGraphParamList),
                self._obj.TimeSpanList, self._obj.GraphValueList, self._obj.OHLCValueList]

    def tuple_to_obj(self, t):
        if len(t) >= 21:
            self._obj.ChartID = t[0]
            self._obj.Title = t[1]
            self._obj.Height = t[2]
            self._obj.Width = t[3]
            self._obj.IsSaveGeometry = t[4]
            self._obj.ReplotTime = t[5]
            self._obj.GlobalTimeAxisParam = t[6]
            self._obj.GlobalValueAxisParam = t[7]
            self._obj.GlobalPlotParam = t[8]
            self._obj.GlobalLegendItemParam = t[9]
            self._obj.PlotParamList = t[10]
            self._obj.ValueAxisParamList = t[11]
            self._obj.LegendItemParamList = t[12]
            self._obj.LineGraphParamList = t[13]
            self._obj.FinancialGraphParamList = t[14]
            self._obj.BarGraphParamList = t[15]
            self._obj.TextGraphParamList = t[16]
            self._obj.MarkerGraphParamList = t[17]
            self._obj.TimeSpanList = t[18]
            self._obj.GraphValueList = t[19]
            self._obj.OHLCValueList = t[20]

            return True
        return False



from ...interface import IData
from ...packer.market.market_param_data_packer import MarketParamDataPacker


class MarketParamData(IData):
    def __init__(self, market_names: str = ''):
        super().__init__(MarketParamDataPacker(self))
        self._MarketNames = market_names

    @property
    def MarketNames(self):
        return self._MarketNames

    @MarketNames.setter
    def MarketNames(self, value: str):
        self._MarketNames = value



from ...interface import IData
from ...packer.trade.account_data_packer import AccountDataPacker


class AccountData(IData):
    def __init__(self, account_id: str = '', pre_balance: float = 0.0, pre_credit: float = 0.0, pre_mortgage: float = 0.0, mortgage: float = 0.0, withdraw: float = 0.0, deposit: float = 0.0,
                 static_balance: float = 0, close_profit: float = 0.0, position_profit: float = 0.0, commission: float = 0.0, balance: float = 0.0, curr_margin: float = 0.0,
                 maintenance_margin: float = 0.0, delivery_margin: float = 0.0, frozen_margin: float = 0.0, frozen_commission: float = 0.0, frozen_cash: float = 0.0, creditlimit: float = 0.0,
                 available: float = 0.0, royalty_in: float = 0.0, royalty_out: float = 0.0, frozen_royalty: float = 0.0, order_commission: float = 0.0, royalty_position_profit: float = 0.0,
                 market_balance: float = 0.0, market_risk_degree: float = 0.0, risk_degree: float = 0.0, currency_no: str = ''):
        super().__init__(AccountDataPacker(self))
        self._AccountID: str = account_id
        self._PreBalance: float = pre_balance
        self._PreCredit: float = pre_credit
        self._PreMortgage: float = pre_mortgage
        self._Mortgage: float = mortgage
        self._Withdraw: float = withdraw
        self._Deposit: float = deposit
        self._StaticBalance: float = static_balance
        self._CloseProfit: float = close_profit
        self._PositionProfit: float = position_profit
        self._Commission: float = commission
        self._Balance: float = balance
        self._CurrMargin: float = curr_margin
        self._MaintenanceMargin: float = maintenance_margin
        self._DeliveryMargin: float = delivery_margin
        self._FrozenMargin: float = frozen_margin
        self._FrozenCommission: float = frozen_commission
        self._FrozenCash: float = frozen_cash
        self._CreditLimit: float = creditlimit
        self._Available: float = available
        self._RoyaltyIn: float = royalty_in
        self._RoyaltyOut: float = royalty_out
        self._FrozenRoyalty: float = frozen_royalty
        self._OrderCommission: float = order_commission
        self._RoyaltyPositionProfit: float = royalty_position_profit
        self._MarketBalance: float = market_balance
        self._MarketRiskDegree: float = market_risk_degree
        self._RiskDegree: float = risk_degree
        self._CurrencyNo: str = currency_no

    @property
    def AccountID(self):
        return self._AccountID

    @AccountID.setter
    def AccountID(self, value: str):
        self._AccountID = value

    @property
    def PreBalance(self):
        return self._PreBalance

    @PreBalance.setter
    def PreBalance(self, value: float):
        self._PreBalance = value

    @property
    def PreCredit(self):
        return self._PreCredit

    @PreCredit.setter
    def PreCredit(self, value: float):
        self._PreCredit = value

    @property
    def PreMortgage(self):
        return self._PreMortgage

    @PreMortgage.setter
    def PreMortgage(self, value: float):
        self._PreMortgage = value

    @property
    def Mortgage(self):
        return self._Mortgage

    @Mortgage.setter
    def Mortgage(self, value: float):
        self._Mortgage = value

    @property
    def Withdraw(self):
        return self._Withdraw

    @Withdraw.setter
    def Withdraw(self, value: float):
        self._Withdraw = value

    @property
    def Deposit(self):
        return self._Deposit

    @Deposit.setter
    def Deposit(self, value: float):
        self._Deposit = value

    @property
    def StaticBalance(self):
        return self._StaticBalance

    @StaticBalance.setter
    def StaticBalance(self, value: float):
        self._StaticBalance = value

    @property
    def CloseProfit(self):
        return self._CloseProfit

    @CloseProfit.setter
    def CloseProfit(self, value: float):
        self._CloseProfit = value

    @property
    def PositionProfit(self):
        return self._PositionProfit

    @PositionProfit.setter
    def PositionProfit(self, value: float):
        self._PositionProfit = value

    @property
    def Commission(self):
        return self._Commission

    @Commission.setter
    def Commission(self, value: float):
        self._Commission = value

    @property
    def Balance(self):
        return self._Balance

    @Balance.setter
    def Balance(self, value: float):
        self._Balance = value

    @property
    def CurrMargin(self):
        return self._CurrMargin

    @CurrMargin.setter
    def CurrMargin(self, value: float):
        self._CurrMargin = value

    @property
    def MaintenanceMargin(self):
        return self._MaintenanceMargin

    @MaintenanceMargin.setter
    def MaintenanceMargin(self, value: float):
        self._MaintenanceMargin = value

    @property
    def DeliveryMargin(self):
        return self._DeliveryMargin

    @DeliveryMargin.setter
    def DeliveryMargin(self, value: float):
        self._DeliveryMargin = value

    @property
    def FrozenMargin(self):
        return self._FrozenMargin

    @FrozenMargin.setter
    def FrozenMargin(self, value: float):
        self._FrozenMargin = value

    @property
    def FrozenCommission(self):
        return self._FrozenCommission

    @FrozenCommission.setter
    def FrozenCommission(self, value: float):
        self._FrozenCommission = value

    @property
    def FrozenCash(self):
        return self._FrozenCash

    @FrozenCash.setter
    def FrozenCash(self, value: float):
        self._FrozenCash = value

    @property
    def CreditLimit(self):
        return self._CreditLimit

    @CreditLimit.setter
    def CreditLimit(self, value: float):
        self._CreditLimit = value

    @property
    def Available(self):
        return self._Available

    @Available.setter
    def Available(self, value: float):
        self._Available = value

    @property
    def RoyaltyIn(self):
        return self._RoyaltyIn

    @RoyaltyIn.setter
    def RoyaltyIn(self, value: float):
        self._RoyaltyIn = value

    @property
    def RoyaltyOut(self):
        return self._RoyaltyOut

    @RoyaltyOut.setter
    def RoyaltyOut(self, value: float):
        self._RoyaltyOut = value

    @property
    def FrozenRoyalty(self):
        return self._FrozenRoyalty

    @FrozenRoyalty.setter
    def FrozenRoyalty(self, value: float):
        self._FrozenRoyalty = value

    @property
    def OrderCommission(self):
        return self._OrderCommission

    @OrderCommission.setter
    def OrderCommission(self, value: float):
        self._OrderCommission = value

    @property
    def RoyaltyPositionProfit(self):
        return self._RoyaltyPositionProfit

    @RoyaltyPositionProfit.setter
    def RoyaltyPositionProfit(self, value: float):
        self._RoyaltyPositionProfit = value

    @property
    def MarketBalance(self):
        return self._MarketBalance

    @MarketBalance.setter
    def MarketBalance(self, value: float):
        self._MarketBalance = value

    @property
    def MarketRiskDegree(self):
        return self._MarketRiskDegree

    @MarketRiskDegree.setter
    def MarketRiskDegree(self, value: float):
        self._MarketRiskDegree = value

    @property
    def RiskDegree(self):
        return self._RiskDegree

    @RiskDegree.setter
    def RiskDegree(self, value: float):
        self._RiskDegree = value

    @property
    def CurrencyNo(self):
        return self._CurrencyNo

    @CurrencyNo.setter
    def CurrencyNo(self, value: str):
        self._CurrencyNo = value



from typing import Dict
from ...interface import IData
from ...packer.chart.line_graph_param_data_packer import LineGraphParamDataPacker


class LineGraphParamData(IData):
    def __init__(self, id: str = '', name: str = '', style: int = 1,
                 color: str = 'white', width: int = 1, plot_index: int = 0, value_axis_id: int = -1,
                 price_tick: float = -1.0, tick_valid_mul: float = -1.0, show_legend: bool = True,
                 legend_format: str = '', legend_color: str = '', bind_ins_id: str = '', bind_range: str = '',
                 join_value_axis=True, valid_max_value: float = 9999999.99, valid_min_value: float = -9999999.99,
                 user_data: Dict[str, str] = {}, visible: bool = True):
        """线图配置参数

        Args:
            id (str, optional): 图形编码. Defaults to ''.
            name (str, optional): 图形名称. Defaults to ''.
            style (int, optional): 图形样式.1.实线 2.线状虚线 3.点状虚线 4.点线间隔线 5.点点线间隔线 7.只显示数据点,没有线 Defaults to 1.
            color (str, optional): 颜色. Defaults to 'white'.
            width (int, optional): 宽度. Defaults to 1.
            plot_index (int, optional): 所属图块. Defaults to 0.
            value_axis_id (int, optional): 所属值轴. Defaults to -1.
            price_tick (float, optional): 变动值. Defaults to -1.0.
            tick_valid_mul (float, optional): 有效倍数. Defaults to -1.0.
            show_legend (bool, optional): 是否显示标签. Defaults to True.
            legend_format (str, optional): 标签格式. Defaults to ''.
            legend_color (str, optional): 标签颜色,默认与图颜色相同. Defaults to ''.
            bind_ins_id (str, optional): 绑定合约编码. Defaults to ''.
            bind_range (str, optional): 绑定合约周期. Defaults to ''.
            join_value_axis (bool, optional): 是否加入值轴可视范围计算内. Defaults to True.
            valid_max_value (float, optional): 最大有效值. Defaults to 9999999.99.
            valid_min_value (float, optional): 最小有效值. Defaults to -9999999.99.
            user_data (Dict[str, str], optional): 用户自定义数据. Defaults to {}.
            visible (bool, optional): 是否显示 Defaults to True.
        """
        super().__init__(LineGraphParamDataPacker(self))
        self._ID: str = id
        self._Name: str = name
        if self._Name == '':
            self._Name = self._ID
        self._Style: int = style
        self._Color: str = color
        self._Width: int = width
        self._PlotIndex: int = plot_index
        self._ValueAxisID: int = value_axis_id
        self._PriceTick: float = price_tick
        self._TickValidMul: float = tick_valid_mul
        self._ShowLegend: bool = show_legend
        self._LegendFormat: str = legend_format
        self._LegendColor: str = legend_color
        self._BindInsID: str = bind_ins_id
        self._BindRange: str = bind_range
        self._JoinValueAxis: bool = join_value_axis
        self._ValidMaxValue: float = valid_max_value
        self._ValidMinValue: float = valid_min_value
        self._UserData: Dict[str, str] = user_data
        self._Visible: bool = visible

    @property
    def ID(self):
        return self._ID

    @ID.setter
    def ID(self, value: str):
        self._ID = value

    @property
    def Name(self):
        return self._Name

    @Name.setter
    def Name(self, value: str):
        self._Name = value

    @property
    def Style(self):
        return self._Style

    @Style.setter
    def Style(self, value):
        self._Style = int(value)

    @property
    def Color(self):
        return self._Color

    @Color.setter
    def Color(self, value):
        self._Color = str(value)

    @property
    def Width(self):
        return self._Width

    @Width.setter
    def Width(self, value):
        self._Width = int(value)

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value):
        self._PlotIndex = int(value)

    @property
    def ValueAxisID(self):
        return self._ValueAxisID

    @ValueAxisID.setter
    def ValueAxisID(self, value):
        self._ValueAxisID = int(value)

    @property
    def PriceTick(self):
        return self._PriceTick

    @PriceTick.setter
    def PriceTick(self, value):
        self._PriceTick = float(value)

    @property
    def TickValidMul(self):
        return self._TickValidMul

    @TickValidMul.setter
    def TickValidMul(self, value):
        self._TickValidMul = float(value)

    @property
    def ShowLegend(self):
        return self._ShowLegend

    @ShowLegend.setter
    def ShowLegend(self, value):
        self._ShowLegend = (value)

    @property
    def LegendFormat(self):
        return self._LegendFormat

    @LegendFormat.setter
    def LegendFormat(self, value):
        self._LegendFormat = str(value)

    @property
    def LegendColor(self):
        return self._LegendColor

    @LegendColor.setter
    def LegendColor(self, value):
        self._LegendColor = str(value)

    @property
    def BindInsID(self):
        return self._BindInsID

    @BindInsID.setter
    def BindInsID(self, value):
        self._BindInsID = str(value)

    @property
    def BindRange(self):
        return self._BindRange

    @BindRange.setter
    def BindRange(self, value):
        self._BindRange = str(value)

    @property
    def JoinValueAxis(self):
        return self._JoinValueAxis

    @JoinValueAxis.setter
    def JoinValueAxis(self, value: bool):
        self._JoinValueAxis = value

    @property
    def ValidMaxValue(self):
        return self._ValidMaxValue

    @ValidMaxValue.setter
    def ValidMaxValue(self, value):
        self._ValidMaxValue = float(value)

    @property
    def ValidMinValue(self):
        return self._ValidMinValue

    @ValidMinValue.setter
    def ValidMinValue(self, value):
        self._ValidMinValue = float(value)

    @property
    def UserData(self):
        return self._UserData

    @UserData.setter
    def UserData(self, value: Dict[str, str]):
        self._UserData = value

    @property
    def Visible(self):
        return self._Visible

    @JoinValueAxis.setter
    def Visible(self, value: bool):
        self._Visible = value




from ...interface import IPacker


class TimeSpanGVListDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [self._obj.TimeSpanList, self._obj.GraphValueList, self._obj.OHLCValueList]

    def tuple_to_obj(self, t):
        if len(t) >= 3:
            self._obj.TimeSpanList = t[0]
            self._obj.GraphValueList = t[1]
            self._obj.OHLCValueList = t[2]

            return True
        return False



from ...interface import IData
from ...packer.market.tick_data_packer import TickDataPacker


class TickData(IData):
    def __init__(self, instrument_id: str = '', exchange_id: str = '', trading_day: str = '', action_day: str = '',
                 trading_time: str = '', last_price: float = 0.0, last_volume: int = 0, ask_price: float = 0.0,
                 ask_volume: int = 0, bid_price: float = 0.0, bid_volume: int = 0):
        super().__init__(TickDataPacker(self))
        self._InstrumentID: str = instrument_id
        self._ExchangeID: str = exchange_id
        self._TradingDay: str = trading_day
        self._ActionDay: str = action_day
        self._TradingTime: str = trading_time
        self._LastPrice: float = last_price
        self._LastVolume: int = last_volume
        self._AskPrice: float = ask_price
        self._AskVolume: int = ask_volume
        self._BidPrice: float = bid_price
        self._BidVolume: int = bid_volume

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def TradingDay(self):
        return self._TradingDay

    @TradingDay.setter
    def TradingDay(self, value: str):
        self._TradingDay = value

    @property
    def ActionDay(self):
        return self._ActionDay

    @ActionDay.setter
    def ActionDay(self, value: str):
        self._ActionDay = value

    @property
    def TradingTime(self):
        return self._TradingTime

    @TradingTime.setter
    def TradingTime(self, value: str):
        self._TradingTime = value

    @property
    def LastPrice(self):
        return self._LastPrice

    @LastPrice.setter
    def LastPrice(self, value: float):
        self._LastPrice = value

    @property
    def LastVolume(self):
        return self._LastVolume

    @LastVolume.setter
    def LastVolume(self, value: int):
        self._LastVolume = value

    @property
    def AskPrice(self):
        return self._AskPrice

    @AskPrice.setter
    def AskPrice(self, value: float):
        self._AskPrice = value

    @property
    def AskVolume(self):
        return self._AskVolume

    @AskVolume.setter
    def AskVolume(self, value: int):
        self._AskVolume = value

    @property
    def BidPrice(self):
        return self._BidPrice

    @BidPrice.setter
    def BidPrice(self, value: float):
        self._BidPrice = value

    @property
    def BidVolume(self):
        return self._BidVolume

    @BidVolume.setter
    def BidVolume(self, value: int):
        self._BidVolume = value
